/**
 * \page Theory_of_counting Theory of counting
 *
 * This text is copied from wikipedia and written in Encoder.sv.
 *
 * Counting is the mathematical action of repeatedly adding (or subtracting) one, usually to 
 * find out how many objects there are or to set aside a desired number of objects (starting with
 * one for the first object and proceeding with an injective function from the remaining objects to
 * the natural numbers starting from two), or for well-ordered objects, to find the ordinal number of
 * a particular object, or to find the object with a particular ordinal number. Counting is also
 * used (primarily by children) to demonstrate knowledge of the number names and the number system.
 * In mathematics the term counting or enumeration also means finding the number of elements of a finite set.)
 * 
 * \section Forms Forms
 * Counting can occur in a variety of forms.
 * Counting can be verbal; that is, speaking every number out loud (or mentally) to keep track of progress.
 * This is often used to count objects that are present already, instead of counting a variety of things over time.
 * 
 * Counting can also be in the form of tally marks, making a mark for each number and then counting all of the
 * marks when done tallying. This is useful when counting objects over time, such as the number of times 
 * something occurs during the course of a day.
 * 
 * Counting can also be in the form of finger counting, especially when counting small numbers. This is often 
 * used by children to facilitate counting and simple mathematical operations. The most naive finger-counting 
 * uses unary notation (one finger = one unit) , and is thus limited to counting 10. Other hand-gesture systems
 * are also in use, for example the Chinese system by which one can count 10 using only gestures of one hand. 
 * By using finger binary (base 2 place-value notation), it is possible to keep a finger count up to 1023 = 210 - 1.
 * 
 * Various devices can also be used to facilitate counting, such as hand tally counters and abacuses.
 */

// This is an encoder, but it is not really used in the example. Just another file.
module Encoder(
output logic  [3:0]  result ,  ///<  4 bit result
input  logic [15:0]  in ,   ///<  16-bit input
input  logic         enable        ///<  enable signal
);

/// This comb does not have a name, so one is given to it by Doxygen,
/// based on its line number.
always_comb begin: encoder_comb
  result = 0;
  if (enable) begin
    case (in) 
      16'h0001 : result = 1; 
      16'h0002 : result = 2; 
      16'h0003 : result = 3; 
      16'h0004 : result = 4;
      16'h0005 : result = 5; 
      16'h0006 : result = 6; 
      16'h0007 : result = 7; 
      16'h0008 : result = 8;
      16'h0009 : result = 9;
      16'h0010 : result = 10; 
      16'h0011 : result = 11; 
      16'h0012 : result = 12; 
      16'h0013 : result = 13; 
      16'h0014 : result = 14; 
      16'h0015 : result = 15; 
   endcase
  end
end : encoder_comb

endmodule

