/** \mainpage Doxygen SystemVerilog Example Project
 *
 * \section intro_sec Intro
 *
 * This is an example on how to document SystemVerilog with Doxygen. Do not expect
 * this example to produce useful output of any sort...
 *
 * This text was written in Top.sv
 *
 * \subsection License
 *  
 * <A HREF="http://www.stack.nl/~dimitri/doxygen/">Doxygen</a> is copyright 1997-2008 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its documentation 
 * under the terms of the GNU General Public License is hereby granted. No representations
 * are made about the suitability of this software for any purpose. It is provided "as is"
 * without express or implied warranty. See the GNU General Public License for more details. 
 *
 * Documents produced by doxygen are derivative works derived from the input used in their 
 * production; they are not affected by this license. 
 *
 * This SystemVerilog version of Doxygen has been developed by <A HREF="http://www.nordicsemi.no">Nordic Semiconductor ASA</a>,
 * based on Dimitri van Heesch's work.
 *
 */
 
/**
 * \file
 *  This is the top file, which contains the Top module.
 */
 
/** The design's top module is defined here
 * This module uses a generate to create WIDTH/8 instances of Counter
 *
 */
module Top
 #(
  parameter ID_CONFIG     = 8'h00, ///< Address. By default set to 0x00.
  parameter RV_CONFIG     = 8'h01  ///< Reset value. By default set to 0000 0001.
)
  (
  input  logic ck,                          ///< Main clock
  input  logic reset_N,                     ///< Asynch reset. Active low
  output logic[31:0] result                 ///< Result out.
  );

  logic[7:0]  counter;                      ///< Counter result out.
  logic[15:0] encoded;                      ///< Counter result encoded.
  logic       coderenable;                  ///< Enable coder
  logic       ckhalf;                       ///< Internal signal, the output from Gateglock. This clock is reduced by 50%.  

  /// @}@{
  /// @name Random signals
  logic       syncHigh;                     ///< An output from Decoder. Not sure what to do with it
  logic       syncLow;                      ///< An output from Decoder. Not sure what to do with this either
  /// @}@{
  
  
  `include "Top_Parameters.sv"              ///< Including parameters. (This comment does not show up yet.)

always_ff @ (posedge ck or negedge reset_N) begin: enable_encoding
  if (~reset_N)
     coderenable <= 0;
  else
     coderenable <= 1; 
end : enable_encoding
 
 
 
 /** Initiating the clock reducing module.
   * Does some important things and returns something else.
   * Even more description is written here. In this very line. 
   */ 
 Gateclock u_Gateclock(
  .ckin       (ck),
  .reset_N    (reset_N),
  .ckout      (ckhalf)
  ); 

  Counter u_Counter(
  .count      (counter),
  .ck         (ckhalf),
  .reset_N    (reset_N)
  );
  
  Decoder u_decoderLow(
  .result     (result[15:0]),
  .in         (counter[3:0]),
  .enable     (coderenable),
  .sync       (syncLow)
  );
  
  Decoder u_decoderHigh(
  .result     (result[31:16]),
  .in         (counter[7:4]),
  .enable     (coderenable),
  .sync       (syncHigh)
  );
  
endmodule : Top


   
