/******************************************************************************
 *
 * 
 *
 *
 * Copyright (C) 1997-2007 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */

%{

#include "genexp.h"
#include "svvalue.h" 
#include "gen_parse.h" // generated header file

#define YY_NEVER_INTERACTIVE 1

QCString    g_genStrToken;
SVValue    g_genResultValue;
int         g_genExpLineNr;
QCString    g_genExpFileName;

static const char *g_inputString;
static int         g_inputPosition;

#undef  YY_INPUT
#define YY_INPUT(buf,result,max_size) result=yyread(buf,max_size);


static int yyread(char *buf,int max_size)
{
  int c=0;
  while( c < max_size && g_inputString[g_inputPosition] )
  {
    *buf = g_inputString[g_inputPosition++] ;
    c++; buf++;
  }
  return c;
}

%}

TIMEUNIT		("s"|"ms"|"us"|"ns"|"ps"|"fs"|"step")

%option nounput

%%

"?"										{ return TOK_QUESTIONMARK; }
":"										{ return TOK_COLON; }
"||"									{ return TOK_OR; }
"&&"									{ return TOK_AND; }
"|"										{ return TOK_BITWISEOR; }
"^"										{ return TOK_BITWISEXOR; }
"^~"|"~^"								{ return TOK_BITWISEXNOR; }
"&"										{ return TOK_AMPERSAND; }
"~&"									{ return TOK_REDUCTIONNAND; }
"~|"									{ return TOK_REDUCTIONOR; }
"!="            						{ return TOK_NOTEQUAL; }
"=="            						{ return TOK_COMPAREEQUAL; }
"="										{ return TOK_EQUAL; }
"<"             						{ return TOK_LESSTHAN; }
">"             						{ return TOK_GREATERTHAN; }
"<="            						{ return TOK_LESSTHANOREQUALTO; }
">="            						{ return TOK_GREATERTHANOREQUALTO; }
"<<"            						{ return TOK_SHIFTLEFT; }
">>"            						{ return TOK_SHIFTRIGHT; }
"<<<"									{ return TOK_ARITHSHIFTLEFT; }
">>>"									{ return TOK_ARITHSHIFTRIGHT; }
"+"             						{ return TOK_PLUS; }
"-"             						{ return TOK_MINUS; }
"*"             						{ return TOK_STAR; }
"/"             						{ return TOK_DIVIDE; }
"%"             						{ return TOK_MOD; }
"~"             						{ return TOK_ONESCOMPLEMENT; }
"!"             						{ return TOK_NOT; }
"==="									{ return TOK_FOURSTATEEQUAL; }
"!=="									{ return TOK_FOURSTATENOTEQUAL; }
"=?="									{ return TOK_WILDEQUAL; }
"!?="									{ return TOK_WILDNOTEQUAL; }
"**"									{ return TOK_EXP; }
"("             						{ return TOK_LPAREN; }
")"             						{ return TOK_RPAREN; }
","										{ return TOK_COMMA; }
"if"									{ return TOK_IF; }
{TIMEUNIT}								{	return TOK_TIMEUNIT; }



"'0"|"'1"|"'x"|"'X"|"'z"|"'Z"			{
						return TOK_UNBASEDUNSIZED;
					}
([1-9][0-9]*)?"'"[sS]?[bB][ \t]*[0-1xXzZ?][0-1xXzZ?_]*         { 
						g_genStrToken=yytext+2; 
  					return TOK_BINARYNUMBER; 
				  }
([1-9][0-9]*)?"'"[sS]?[oO][ \t]*[0-7xXzZ?][0-7xXzZ?_]*         { 
						g_genStrToken=yytext+2; 
  					return TOK_OCTALNUMBER; 
				  }
([1-9][0-9]*)?"'"[sS]?[dD][ \t]*(([0-9][0-9_]*)|([xXzZ?]"_"*))         { 
						g_genStrToken=yytext+2; 
  					return TOK_DECIMALNUMBER; 
				  }
[0-9][0-9_]*          { 
						g_genStrToken=yytext;
  					return TOK_UNSIGNEDNUMBER; 
				  }
[0-9][0-9_]*"."[0-9][0-9_]* { 
						g_genStrToken=yytext;
  					return TOK_REALNUMBER; 
				  }
[0-9][0-9_]*("."[0-9][0-9_]*)?[eE][+-]?[0-9][0-9_]* { 
						g_genStrToken=yytext;
  					return TOK_REALEXPNUMBER; 
					}
   
([1-9][0-9]*)?"'"[sS]?[hH][ \t]*[0-9a-fA-FxXzZ?][0-9a-fA-FxXzZ?_]*  { 
						g_genStrToken=yytext+2; 
            return TOK_HEXADECIMALNUMBER;
					}
[a-z_A-Z][a-z_A-Z0-9$]* {
						return TOK_ID;
					}

.					{
					}		   
\n		
["].*["]	{
						return TOK_STRINGLITERAL;	
					}

%%

bool parseGenExpression(const char *fileName,int lineNr,const QCString &s)
{
  //printf("Expression: `%s'\n",s.data());
  g_genExpFileName = fileName;
  g_genExpLineNr = lineNr;
  g_inputString = s;
  g_inputPosition = 0;
  genExpYYrestart( genExpYYin );
  genExpYYparse();
  //printf("Result: %ld\n",(long)g_genResultValue);
  return (long)g_genResultValue!=0;
}
extern "C" {
  int genExpYYwrap() { return 1; }
}
