/******************************************************************************
 *
 * 
 *
 *
 * Copyright (C) 1997-2007 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */

%{

#include "svvalue.h"
#include "genexp.h"
#include "message.h"

#if defined(_MSC_VER)
#define MSDOS
#endif
#define YYDEBUG 0
#define YYSTYPE SVValue

#include <stdio.h>
#include <stdlib.h>

int genExpYYerror(const char *s)
{
  warn(g_genExpFileName,g_genExpLineNr,
       "Problem during constant expression evaluation: %s",s);
  g_genResultValue = SVValue((long)0);
  return 0;
}


int genExpYYlex();

%}

%token TOK_QUESTIONMARK
%token TOK_COLON
%token TOK_OR
%token TOK_AND
%token TOK_BITWISEOR
%token TOK_BITWISEXOR
%token TOK_AMPERSAND
%token TOK_NOTEQUAL
%token TOK_EQUAL
%token TOK_LESSTHAN
%token TOK_GREATERTHAN
%token TOK_LESSTHANOREQUALTO
%token TOK_GREATERTHANOREQUALTO
%token TOK_SHIFTLEFT
%token TOK_SHIFTRIGHT
%token TOK_PLUS
%token TOK_MINUS
%token TOK_STAR
%token TOK_DIVIDE
%token TOK_MOD
%token TOK_ONESCOMPLEMENT
%token TOK_NOT
%token TOK_LPAREN
%token TOK_RPAREN
%token TOK_OCTALNUMBER
%token TOK_DECIMALNUMBER
%token TOK_UNSIGNEDNUMBER
%token TOK_HEXADECIMALNUMBER
%token TOK_BINARYNUMBER
%token TOK_REALNUMBER
%token TOK_REALEXPNUMBER
%token TOK_ID
%token TOK_IF
%token TOK_ESCAPEDID
%token TOK_COMMA
%token TOK_COMPAREEQUAL
%token TOK_FOURSTATEEQUAL
%token TOK_FOURSTATENOTEQUAL
%token TOK_WILDEQUAL
%token TOK_WILDNOTEQUAL
%token TOK_EXP
%token TOK_BITWISEXNOR
%token TOK_ARITHSHIFTLEFT
%token TOK_ARITHSHIFTRIGHT
%token TOK_TIMEUNIT
%token TOK_UNBASEDUNSIZED
%token TOK_STRINGLITERAL
%token TOK_REDUCTIONNAND
%token TOK_REDUCTIONOR

%nonassoc TOK_EQUAL
%right TOK_QUESTIONMARK TOK_COLON
%left TOK_OR
%left TOK_AND
%left TOK_BITWISEOR
%left TOK_BITWISEXOR TOK_BITWISEXNOR
%left TOK_AMPERSAND
%left TOK_COMPAREEQUAL TOK_NOTEQUAL TOK_FOURSTATEEQUAL TOK_FOURSTATENOTEQUAL TOK_WILDEQUAL TOK_WILDNOTEQUAL
%left TOK_LESSTHAN TOK_LESSTHANOREQUALTO TOK_GREATERTHAN TOK_GREATERTHANOREQUALTO
%left TOK_SHIFTLEFT TOK_SHIFTRIGHT TOK_ARITHSHIFTLEFT TOK_ARITHSHIFTRIGHT
%left TOK_PLUS TOK_MINUS
%left TOK_STAR TOK_DIVIDE TOK_MOD
%left TOK_EXP


%%

start:											TOK_IF TOK_LPAREN constant_expression TOK_RPAREN {
															g_genResultValue = $3;
															return 0;
														}
;

constant_expression:				constant_primary
														|
														TOK_MINUS constant_primary { $$ = SVValue((long)(-(long)$2)); }
														|
														TOK_PLUS constant_primary { $$ = SVValue((long)(+(long)$2)); }
														|
														TOK_NOT constant_primary { $$ = SVValue((long)(!(long)$2)); }
														|
														TOK_ONESCOMPLEMENT constant_primary { $$ = SVValue((long)(~(long)$2)); }
														|
														TOK_AMPERSAND constant_primary { yyerror("Unary reduction operator '&' not supported"); }
														|
														TOK_REDUCTIONNAND constant_primary { yyerror("Unary reduction operator '~&' not supported"); }
														|
														TOK_BITWISEOR constant_primary { yyerror("Unary reduction operator '|' not supported"); }
														|
														TOK_REDUCTIONOR constant_primary { yyerror("Unary reduction operator '~|' not supported"); }
														|
														TOK_BITWISEXOR constant_primary { yyerror("Unary reduction operator '^' not supported"); }
														|
														TOK_BITWISEXNOR constant_primary { yyerror("Unary reduction operators '~^' and '^~' not supported"); }
														|
														TOK_MINUS attribute_instances constant_primary { $$ = SVValue((long)(-(long)$2)); yyerror("Ignoring attribute(s)"); }
														|
														TOK_PLUS attribute_instances constant_primary { $$ = SVValue((long)(+(long)$2)); yyerror("Ignoring attribute(s)"); }
														|
														TOK_NOT attribute_instances constant_primary { $$ = SVValue((long)(!(long)$2)); yyerror("Ignoring attribute(s)"); }
														|
														TOK_ONESCOMPLEMENT attribute_instances constant_primary { $$ = SVValue((long)(~(long)$2)); yyerror("Ignoring attribute(s)"); }
														|
														TOK_AMPERSAND attribute_instances constant_primary { yyerror("Unary reduction operator '&' not supported"); yyerror("Ignoring attribute(s)"); }
														|
														TOK_REDUCTIONNAND attribute_instances constant_primary { yyerror("Unary reduction operator '~&' not supported"); yyerror("Ignoring attribute(s)"); }
														|
														TOK_BITWISEOR attribute_instances constant_primary { yyerror("Unary reduction operator '|' not supported"); yyerror("Ignoring attribute(s)"); }
														|
														TOK_REDUCTIONOR attribute_instances constant_primary { yyerror("Unary reduction operator '~|' not supported"); yyerror("Ignoring attribute(s)"); }
														|
														TOK_BITWISEXOR attribute_instances constant_primary { yyerror("Unary reduction operator '^' not supported"); yyerror("Ignoring attribute(s)"); }
														|
														TOK_BITWISEXNOR attribute_instances constant_primary { yyerror("Unary reduction operators '~^' and '^~' not supported"); yyerror("Ignoring attribute(s)"); }
														|
														
														constant_expression TOK_COMPAREEQUAL constant_expression { $$ = SVValue( (long)((double)$1 == (double)$3) ); }
														|
														constant_expression TOK_NOTEQUAL constant_expression { $$ = SVValue( (long)((double)$1 != (double)$3) ); }
														|
														constant_expression TOK_LESSTHAN constant_expression { $$ = SVValue( (double)((double)$1 < (double)$3) ); }
														|
														constant_expression TOK_LESSTHANOREQUALTO constant_expression { $$ = SVValue( (double)((double)$1 <= (double)$3) ); }
														|
														constant_expression TOK_GREATERTHAN constant_expression { $$ = SVValue( (double)((double)$1 > (double)$3) ); }
														|
														constant_expression TOK_GREATERTHANOREQUALTO constant_expression { $$ = SVValue( (double)((double)$1 >= (double)$3) ); }
														|
														constant_expression TOK_PLUS constant_expression { $$ = SVValue( (double)((double)$1 + (double)$3) ); }
														|
														constant_expression TOK_MINUS constant_expression { $$ = SVValue( (double)((double)$1 - (double)$3) ); }
														|
														constant_expression TOK_STAR constant_expression { $$ = SVValue( (double)((double)$1 * (double)$3) ); }
														|
														constant_expression TOK_DIVIDE constant_expression { $$ = SVValue( (double)((double)$1 / (double)$3) ); if((double)$3 == 0.0) yyerror("Divide by zero"); }
														|
														constant_expression TOK_MOD constant_expression { $$ = SVValue( (long)((long)$1 % (long)$3) ); }
														|
														constant_expression TOK_EXP constant_expression { $$ = SVValue( pow($1,$3) ); }
														|
														constant_expression TOK_AND constant_expression { $$ = SVValue( (long)((long)$1 && (long)$3) ); }
														|
														constant_expression TOK_OR constant_expression { $$ = SVValue( (long)((long)$1 || (long)$3) ); }
														|
														constant_expression TOK_AMPERSAND constant_expression { $$ = SVValue( (long)((long)$1 & (long)$3) ); }
														|
														constant_expression TOK_BITWISEOR constant_expression { $$ = SVValue( (long)((long)$1 | (long)$3) ); }
														|
														constant_expression TOK_BITWISEXOR constant_expression { $$ = SVValue( (long)((long)$1 ^ (long)$3) ); }
														|
														constant_expression TOK_BITWISEXNOR constant_expression { $$ = SVValue( (long)((long)$1 ^~ (long)$3) ); }
														|
														constant_expression TOK_SHIFTRIGHT constant_expression { $$ = SVValue( (long)((long)$1 >> (long)$3) ); }
														|
														constant_expression TOK_SHIFTLEFT constant_expression { $$ = SVValue( (long)((long)$1 << (long)$3) ); }
														|
														constant_expression TOK_ARITHSHIFTLEFT constant_expression { yyerror("Arithmetic shift not supported"); }
														|
														constant_expression TOK_ARITHSHIFTRIGHT constant_expression { yyerror("Arithmetic shift not supported"); }
														|
														constant_expression TOK_FOURSTATEEQUAL constant_expression { yyerror("4-State operator not supported"); }
														|
														constant_expression TOK_FOURSTATENOTEQUAL constant_expression { yyerror("4-State operator not supported"); }
														|
														constant_expression TOK_WILDEQUAL constant_expression { yyerror("4-State operator not supported"); }
														|
														constant_expression TOK_WILDNOTEQUAL constant_expression { yyerror("4-State operator not supported"); }
														|

														constant_expression TOK_COMPAREEQUAL attribute_instances constant_expression { $$ = SVValue( (long)((double)$1 == (double)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_NOTEQUAL attribute_instances constant_expression { $$ = SVValue( (long)((double)$1 != (double)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_LESSTHAN attribute_instances constant_expression { $$ = SVValue( (double)((double)$1 < (double)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_LESSTHANOREQUALTO attribute_instances constant_expression { $$ = SVValue( (double)((double)$1 <= (double)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_GREATERTHAN attribute_instances constant_expression { $$ = SVValue( (double)((double)$1 > (double)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_GREATERTHANOREQUALTO attribute_instances constant_expression { $$ = SVValue( (double)((double)$1 >= (double)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_PLUS attribute_instances constant_expression { $$ = SVValue( (double)((double)$1 + (double)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_MINUS attribute_instances constant_expression { $$ = SVValue( (double)((double)$1 - (double)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_STAR attribute_instances constant_expression { $$ = SVValue( (double)((double)$1 * (double)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_DIVIDE attribute_instances constant_expression { $$ = SVValue( (double)((double)$1 / (double)$4) ); if((double)$4 == 0.0) yyerror("Divide by zero"); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_MOD attribute_instances constant_expression { $$ = SVValue( (long)((long)$1 % (long)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_EXP attribute_instances constant_expression { $$ = SVValue( pow($1,$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_AND attribute_instances constant_expression { $$ = SVValue( (long)((long)$1 && (long)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_OR attribute_instances constant_expression { $$ = SVValue( (long)((long)$1 || (long)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_AMPERSAND attribute_instances constant_expression { $$ = SVValue( (long)((long)$1 & (long)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_BITWISEOR attribute_instances constant_expression { $$ = SVValue( (long)((long)$1 | (long)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_BITWISEXOR attribute_instances constant_expression { $$ = SVValue( (long)((long)$1 ^ (long)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_BITWISEXNOR attribute_instances constant_expression { $$ = SVValue( (long)((long)$1 ^~ (long)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_SHIFTRIGHT attribute_instances constant_expression { $$ = SVValue( (long)((long)$1 >> (long)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_SHIFTLEFT attribute_instances constant_expression { $$ = SVValue( (long)((long)$1 << (long)$4) ); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_ARITHSHIFTLEFT attribute_instances constant_expression { yyerror("Arithmetic shift not supported"); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_ARITHSHIFTRIGHT attribute_instances constant_expression { yyerror("Arithmetic shift not supported"); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_FOURSTATEEQUAL attribute_instances constant_expression { yyerror("4-State operator not supported"); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_FOURSTATENOTEQUAL attribute_instances constant_expression { yyerror("4-State operator not supported"); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_WILDEQUAL attribute_instances constant_expression { yyerror("4-State operator not supported"); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_WILDNOTEQUAL attribute_instances constant_expression { yyerror("4-State operator not supported"); yyerror("Ignoring attribute(s)"); }
														|
														constant_expression TOK_QUESTIONMARK constant_expression TOK_COLON constant_expression { if((long)$1) {$$ = (double)$3; } else { $$ = (double)$5;  } }
														|
														constant_expression TOK_QUESTIONMARK attribute_instances constant_expression TOK_COLON constant_expression {  if((long)$1) {$$ = (double)$3; } else { $$ = (double)$5;} yyerror("Ignoring attribute(s)"); }
														|
														TOK_LPAREN constant_expression TOK_RPAREN { $$ = (double)$2; }
;

attribute_instances:				attribute_instance
														|
														attribute_instances attribute_instance
;

attribute_instance:					TOK_LPAREN TOK_STAR attr_name TOK_STAR TOK_RPAREN
														|
														TOK_LPAREN TOK_STAR attr_name TOK_EQUAL constant_expression TOK_STAR TOK_RPAREN
														|
														TOK_LPAREN TOK_STAR attr_name attr_specs TOK_STAR TOK_RPAREN
														|
														TOK_LPAREN TOK_STAR attr_name TOK_EQUAL constant_expression attr_specs TOK_STAR TOK_RPAREN
;

attr_specs:									TOK_COMMA attr_name
														|
														TOK_COMMA attr_name TOK_EQUAL constant_expression
														|
														attr_specs TOK_COMMA attr_name
														|
														attr_specs TOK_COMMA attr_name TOK_EQUAL constant_expression 
;

attr_name:									identifier
;

constant_primary:						primary_literal
;

primary_literal:						number
														|
														time_literal
														|
														TOK_UNBASEDUNSIZED
														|
														TOK_STRINGLITERAL
;

time_literal:								TOK_UNSIGNEDNUMBER TOK_TIMEUNIT
														|
														TOK_REALNUMBER TOK_TIMEUNIT
;

number:											integral_number
														|
														real_number
;

real_number:								TOK_REALNUMBER { $$ = parseSVFloat(); }
														|
														TOK_REALEXPNUMBER { $$ = parseSVFloat(); }
;

integral_number:						TOK_DECIMALNUMBER { $$ = parseSVDecimal(); }
														|
														TOK_UNSIGNEDNUMBER { $$ = parseSVDecimal(); }
														|
														TOK_OCTALNUMBER { $$ = parseSVOctal(); }
														|
														TOK_BINARYNUMBER { $$ = parseSVBinary(); }
														|
														TOK_HEXADECIMALNUMBER { $$ = parseSVHexadecimal(); }
;

identifier:									simple_identifier
														|
														escaped_identifier
;

simple_identifier:					TOK_ID
;

escaped_identifier:					TOK_ESCAPEDID
;

%%
