/*****************************************************************************
 *
 * 
 *
 * Copyright (C) 1997-2007 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */
 
%{
	/* Comment */

%}
   
%{

/*
 *	includes
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>

#include "qtbc.h"
#include <qarray.h>
#include <qstack.h>
#include <qregexp.h>
#include <unistd.h>
#include <qfile.h>
#include <qmap.h>
  
#include "svscanner.h"
#include "entry.h"
#include "message.h"
#include "config.h"
#include "doxygen.h"
#include "util.h"
#include "defargs.h"
#include "language.h"
#include "commentscan.h"
#include "svcode.h"
#include "genexp.h"


#define YY_NEVER_INTERACTIVE 1

/* -----------------------------------------------------------------
 *
 *	statics
 */
static ParserInterface	*g_thisParser;
static const char *     inputString;
static int							inputPosition;
static QFile            inputFile;
static int							lastContext;
static int							lastCContext;
static int              lastCPPContext;
static int              lastStringContext;
static int              lastCurlyContext;
static int              lastRoundContext;
static int              lastInitializerContext;
static int              lastPreLineCtrlContext;
static int              lastSkipVerbStringContext;
static int              lastCommentInArgContext;

static Protection				protection;
static Protection				baseProt;
static int							roundCount   = 0 ;
static int							beginCount   = 0 ;
static int							curlyCount   = 0 ;
static int							squareCount  = 0 ;
static int							elseBeginCount = 0;

static Entry*						current_root = 0 ;
static Entry*						current      = 0 ;
static Entry*						previous     = 0 ;
static Entry*						tempEntry    = 0 ;
static Entry*						firstTypedefEntry = 0 ;

static int							yyLineNr     = 1 ;
static int              anonCount    = 0 ;        
static int              anonNSCount  = 0 ;        
static QCString					yyFileName;
static MethodTypes 			mtype;
static bool    					gstat;
static Specifier 				virt;
static Specifier 				baseVirt;
static QCString         msType,msName,msArgs;
static bool             isTypedef;
static QCString         baseName;

static bool							isParameter = FALSE;

static int              argRoundCount;
static int              currentArgumentContext;
static int              lastCopyArgStringContext;
static int              lastCopyArgContext;
static QCString         *copyArgString;
static QCString         fullArgString;

static char             lastCopyArgChar;

static QGString         *pCopyCurlyGString;
static QGString         *pCopyRoundGString;
static QGString         *pCopyQuotedGString;
static QGString         *pSkipVerbString;

static bool             insideCode;
static bool             needsSemi;
static bool							insideParameterList;
static bool							insideModport = FALSE;
static bool							skipSignal = FALSE;
static bool							hasDoneAlwaysParam = FALSE;

static int  						initBracketCount;

static bool             g_inputFromFile;

static int              docBlockContext;
static QCString         docBlock;
static QCString         docBlockName;
static bool             docBlockInBody;
static bool             docBlockAutoBrief;
static char             docBlockTerm;

static QCString					svStatement;
static QCString					oldName;
static QCString					oldProgram;
static QCString					oldArgs;
static QCString         fakeName;
struct AlwaysInfo
{
	QCString alwaysContext;
};
static QStack<AlwaysInfo>	alwaysStack;
static QCString					currentAlwaysContext;

static bool             g_lexInit = FALSE;

struct ParameterInfo
{
  ParameterInfo() {}
  ~ParameterInfo() {}
  QCString fileName;
  QCString parameterName;
  bool operator<( const ParameterInfo &parameterInfo ) const {
    QString s1( parameterInfo.toString() );
    QString s2( toString() );
    return ( s1 < s2 );
  }
  QString toString() const {
    QString s = QString( "%1_%2" ).arg( fileName ).arg( parameterName );
    return QString( s );
  }
};

struct ForLoopInfo
{
	ForLoopInfo() {}
	~ForLoopInfo() {}
	QCString variableName;
	QCString constantExpression;
	bool increment;
	QString loopContent;
};
static int							beginCountFor   = 0 ;
static QMap<ParameterInfo, QCString> parameterMap;
static QCString					generateString;
static int							generateContext;
static int				moduleCounter;
static QStack<int>			generateStack;
static QStack<ForLoopInfo>		forLoopStack;
static QMap<QCString, int>			moduleMap;
static QCString				covergroupName;



//-----------------------------------------------------------------------------

// forward declarations
//static void handleGroupStartCommand(const char *header);
//static void handleGroupEndCommand();

//-----------------------------------------------------------------------------

static void initParser()
{
  baseName.resize(0);
  protection = Public;
  baseProt = Public;
  roundCount = 0;
  curlyCount = 0;
  beginCount = 0;
  elseBeginCount = 0;
  beginCountFor = 0;
  moduleMap.clear();
  forLoopStack.clear();
  generateStack.clear();  
  mtype = Method;
  gstat = FALSE;
  virt = Normal;
  baseVirt = Normal;
  isTypedef = FALSE;
  insideCode=FALSE;
  previous = 0;
}

static void initEntry()
{
  current->protection = protection ;
  current->systemVerilogType = SystemVerilogNone;
  current->mtype      = mtype;
  current->virt       = virt;
  current->stat       = gstat;
  initGroupInfo(current);
  isTypedef=FALSE;
}


//-----------------------------------------------------------------------------

static void lineCount()
{
  for( const char* c = yytext ; *c ; ++c )
    yyLineNr += (*c == '\n') ;
}

static void addType( Entry* current )
{
    uint tl=current->type.length();
    if( tl>0 && !current->name.isEmpty() && current->type.at(tl-1)!='.') 
    {
      current->type += ' ' ;
    }
    current->type += current->name ;
    current->name.resize(0) ;
    tl=current->type.length();
    if( tl>0 && !current->args.isEmpty() && current->type.at(tl-1)!='.') 
    {
      current->type += ' ' ;
    }
    current->type += current->args ;
    current->args.resize(0) ;
    current->argList->clear();
}


static QCString stripQuotes(const char *s)
{
  QCString name;
  if (s==0 || *s==0) return name;
  name=s;
  if (name.at(0)=='"' && name.at(name.length()-1)=='"')
  {
    name=name.mid(1,name.length()-2);
  }
  return name;
}

QCString replaceVariables(const QCString &expr)
{
	unsigned int i = 0;
	QCString e = expr;
	//QRegExp rx( "\\b(([A-Z]|[a-z]){1}\\w*)\\b" );
	QRegExp rx( "[a-zA-Z_][a-zA-Z0-9]*" );
	//QRegExp rx2( "\\W" );
	QRegExp rx2( "[^a-zA-Z0-9]" );
	//printf("The string: %s\n", e);
	while(i < e.length()) {
		QCString tmpExpr;
		QCString replaceExpr;
		int tmpPos = 0;
		int tmpPos2 = 0;
		tmpPos = e.find(rx,i);
		tmpPos2 = e.find(rx2,tmpPos);
		//printf("Found a variable between character %d and %d\n", tmpPos, tmpPos2);
		tmpExpr = e.mid(tmpPos, tmpPos2-tmpPos);
		if(tmpPos2 > 0){
			i = i + tmpPos2;
		}
		else{
			i++;
		}
		//printf("tmpExpr: %s\n", tmpExpr);
		FileDef	*fileDef;
		bool ambig;
		fileDef = findFileDef(Doxygen::inputNameDict,yyFileName,ambig);
		ParameterInfo *tmpParInfo = new ParameterInfo();
		QCString tmpParameter;

		// Finding included files
		QList<IncludeInfo> *tmpIncludeList = fileDef->includeFileList();
		if(tmpIncludeList != NULL)
		{
			for(unsigned int i = 0; i < tmpIncludeList->count(); i++)
			{
				tmpParInfo->fileName = tmpIncludeList->at(i)->includeName;
				tmpParInfo->parameterName = tmpExpr;
				tmpParameter = parameterMap[*tmpParInfo];
				if(tmpParameter != NULL)
				{
					replaceExpr = tmpParameter;
					break;
				}
			}
		}
		if(tmpParameter == NULL)
		{	
			tmpParInfo->fileName = fileDef->name();
			tmpParInfo->parameterName = tmpExpr;
			tmpParameter = parameterMap[*tmpParInfo];
			if(tmpParameter != NULL)
			{
				replaceExpr = tmpParameter;
			}
			else
			{
				//printf("%s: Warning, could not find parameter '%s' in generate block\n", yyFileName, yytext);						
			}
		}
		if(tmpParameter != NULL && tmpPos != -1 && tmpPos2 != -1){
			e.replace(tmpPos, tmpPos2-tmpPos, replaceExpr);
		}
		//printf("e: %s\n", e);
	}
	return e;
}

bool computeGenerate(const QCString &expr)
{
  QCString e=expr;
  if (e.isEmpty()) return FALSE;
  //printf("parsing `%s'\n",e.data());
  return parseGenExpression(yyFileName,yyLineNr,e);
}

//-----------------------------------------------------------------

static void startCommentBlock(bool);
static void handleCommentBlock(const QCString &doc,bool brief);
static void handleParametersCommentBlocks();

//-----------------------------------------------------------------------------

static void prependScope()
{
  if (current_root->section & Entry::SCOPE_MASK)
  {
    //printf("--- prependScope %s to %s\n",current_root->name.data(),current->name.data());
    current->name.prepend(current_root->name+"::");
    if (current_root->tArgLists)
    {
      if (current->tArgLists==0)
      {
				current->tArgLists = new QList<ArgumentList>;
				current->tArgLists->setAutoDelete(TRUE);
      }
      //printf("prependScope #=%d #current=%d\n",current_root->tArgLists->count(),current->tArgLists->count());
      QListIterator<ArgumentList> talsi(*current_root->tArgLists);
      ArgumentList *srcAl=0;
      for (talsi.toLast();(srcAl=talsi.current());--talsi)
      {
        ArgumentList *dstAl = new ArgumentList;
				dstAl->setAutoDelete(TRUE);
				QListIterator<Argument> tali(*srcAl);
        Argument *a;
        for (;(a=tali.current());++tali)
        {
          dstAl->append(new Argument(*a));
					//printf("appending argument %s %s\n",a->type.data(),a->name.data());
        }	  
        current->tArgLists->insert(0,dstAl);	
      }
    }
  }
}

//-----------------------------------------------------------------------------
/* ----------------------------------------------------------------- */
#undef	YY_INPUT
#define	YY_INPUT(buf,result,max_size) result=yyread(buf,max_size);

static int yyread(char *buf,int max_size)
{
    int c=0;
    if (g_inputFromFile)
    {
       c = inputFile.readBlock(buf,max_size);
       if (c==-1) yy_fatal_error("input in flex scanner failed");
    }
    else
    {
      while( c < max_size && inputString[inputPosition] )
      {
	*buf = inputString[inputPosition++] ;
	//printf("%d (%c)\n",*buf,*buf);
	c++; buf++;
      }
    }
    return c;
}

%}

       /* start command character */
CMD	  ("\\"|"@")
SECTIONCMD {CMD}("image"|"author"|"internal"|"version"|"date"|"deprecated"|"param"|"exception"|"return"[s]?|"retval"|"bug"|"warning"|"par"|"sa"|"see"|"pre"|"post"|"invariant"|"note"|"remark"[s]?|"todo"|"test"|"xrefitem"|"ingroup"|"callgraph"|"callergraph"|"latexonly"|"htmlonly"|"xmlonly"|"{"|"verbatim"|"dotfile"|"dot"|"defgroup"|"addtogroup"|"weakgroup"|"class"|"namespace"|"union"|"struct"|"fn"|"var"|"details"|"typedef"|"def"|"overload")|("<"{PRE}">")
SVTYPES	  ("logic"|"bit"|"byte"|"shortint"|"int"|"longint"|"shortreal"|"integer"|"genvar"|"wire"|"reg"|"static"|"string"|"event"|"chandle"|"input"|"output"|"inout")
CONSTTYPES ("localparam"|"specparam"|"const")
BN        [ \t\n\r]
BL        [ \t\r]*"\n" 
B         [ \t]
ID        "$"?[a-z_A-Z][a-z_A-Z0-9]*
SCOPENAME "$"?(({ID}?{BN}*"::"{BN}*)*)((~{BN}*)?{ID})
PACKAGESCOPE {ID}"::"({ID}|"*")
PRE       [pP][rR][eE]
CODE      [cC][oO][dD][eE]
CHARLIT   (("'"\\[0-7]{1,3}"'")|("'"\\."'")|("'"[^' \\\n]{1,4}"'"))
TYPEDEFPREFIX (("typedef"{BN}+)?)((("const"){BN}+)?)
LOGICOP "=="|"!="|">"|"<"|">="|"<="|"&&"|"||"|"!"|"==="|"!=="

%option noyywrap

  /* language parsing states */

%x	Define
%x	DefineEnd
%x	CompoundName
%x	ClassVar
%x	FindMembers
%x      FindFields
%x	Function
%x	FuncRound
%x	FuncQual
%x	Array
%x	ReadBody
%x	SkipCurly
%x	SkipCurlyEndDoc
%x      SkipString
%x	SkipCPP
%x	SkipComment
%x	SkipCxxComment
%x	SkipSignalCurly
%x	TypedefName
%x	Comment
%x	PreLineCtrl
%x	SkipVerbString
%x	ReadInitializer
%x  MemberSpec
%x  MemberSpecSkip
%x	ReadFuncArgType
%x	CopyGString
%x	GCopyRound
%x	GCopyCurly
%x      CopyArgString
%x	CopyArgRound
%x	CopyArgComment
%x	CopyArgCommentLine
%x	CopyArgVerbatim

 /** Prototype scanner states */

%x      Prototype
%x      PrototypePtr
%x      PrototypeQual
%x      PrototypeSkipLine

 /** comment parsing states */

%x	DocLine
%x	DocBlock
%x      DocCopyBlock

 /* new SystemVerilog states */

%x	Always
%x	AlwaysCase
%x	AlwaysElse
%x	AlwaysIf
%x	AlwaysMaybeElse
%x	AlwaysName
%x	AlwaysParam
%x	Assign
%x  CoverGroup
%x  CoverGroupEvent
%x  CoverGroupSkip
%x	Enum
%x	EnumName
%x	Generate
%x	GenerateDoElse
%x	GenerateDoFor
%x	GenerateDoIf
%x	GenerateElse
%x	GenerateFor
%x  GenerateForSave
%x	GenerateIf
%x	GenerateSkip
%x	GenerateSkipElse
%x	GetGenerateSignal
%x	GetInclude
%x	GetParameter
%x	GetSignal
%x	ModuleInstName
%x	ModuleInstNet
%x	ModuleInstPort
%x	ModuleNumberSign
%x  Property
%x	ReadEnumBody
%x	ReadPorts
%x	SkipAlwaysIf
%x  SkipAssert
%x  SkipProperty
%x  SvFunction
%x  SvFunctionArgs
%x  SvFunctionBody
%x	SvStruct
%x	SvStructName

%%

<FindMembers>{BN}{1,80}			{
  					lineCount();
  				}				
<FindMembers>{B}*{TYPEDEFPREFIX}("interface"){BN}+ {
						current->insidePortList = TRUE;
						isTypedef=FALSE;
  					current->section = Entry::CLASS_SEC;
            current->spec = Entry::Interface;
					  addType( current ) ;
					  current->type += " interface" ;
					  current->fileName  = yyFileName;
					  current->startLine = yyLineNr;
					  current->bodyLine  = yyLineNr;
					  BEGIN( CompoundName );
  				}	
<FindMembers>{B}*{TYPEDEFPREFIX}("package"){BN}+ {
					  current->insidePortList = TRUE;
					  isTypedef=FALSE;
  					  current->section = Entry::CLASS_SEC;
					  current->spec = Entry::Package;
					  addType( current ) ;
					  current->type += " package" ;
					  current->fileName  = yyFileName;
					  current->startLine = yyLineNr;
					  current->bodyLine  = yyLineNr;
					  BEGIN( CompoundName );
  				}	  					
 <FindMembers>{B}*{TYPEDEFPREFIX}("module"){BN}+ {
					  current->insidePortList = TRUE;
					  current_root->insidePortList = TRUE;
					  isTypedef=((QCString)yytext).find("typedef")!=-1;
  					current->section = Entry::CLASS_SEC;
  					current->spec = Entry::Module;
					  addType( current ) ;
					  current->type += " module" ;
					  current->fileName  = yyFileName;
					  current->startLine = yyLineNr;
					  current->bodyLine  = yyLineNr;
					  lineCount() ;
					  if (yytext[yyleng-1]=='{') unput('{');
					  current->protection = Public;
					  BEGIN( CompoundName );
					}
<FindMembers>{B}*{TYPEDEFPREFIX}("class{") | 
<FindMembers>{B}*{TYPEDEFPREFIX}("class"){BN}+ { 
					  isTypedef=((QCString)yytext).find("typedef")!=-1;
  					current->section = Entry::CLASS_SEC;
					  addType( current ) ;
					  current->type += " class" ;
					  current->fileName  = yyFileName;
					  current->startLine = yyLineNr;
					  current->bodyLine  = yyLineNr;
					  lineCount() ;
					  if (yytext[yyleng-1]=='{') unput('{');
					  BEGIN( CompoundName );
					}
<FindMembers>{B}*{TYPEDEFPREFIX}"struct{" | 
<FindMembers>{B}*{TYPEDEFPREFIX}"struct"/{BN}+ { 
					  current->insidePortList = FALSE;
					  isTypedef=((QCString)yytext).find("typedef")!=-1;
  					current->section = Entry::CLASS_SEC ;
            current->spec    = Entry::Struct;
					  addType( current ) ;
					  current->type += " struct" ;
					  current->fileName  = yyFileName;
					  current->startLine = yyLineNr;
					  current->bodyLine  = yyLineNr;
					  lineCount() ;
					  if (yytext[yyleng-1]=='{') unput('{');
					  BEGIN( SvStruct ) ;
					}
<FindMembers>{B}*{TYPEDEFPREFIX}"union{" |
<FindMembers>{B}*{TYPEDEFPREFIX}"union"{BN}+ { 
					  current->insidePortList = FALSE;
					  isTypedef=((QCString)yytext).find("typedef")!=-1;
  					current->section = Entry::CLASS_SEC;
            current->spec    = Entry::Union;
					  addType( current ) ;
					  current->type += " union" ;
					  current->fileName  = yyFileName;
					  current->startLine = yyLineNr;
					  current->bodyLine  = yyLineNr;
					  lineCount() ;
					  if (yytext[yyleng-1]=='{') unput('{');
					  BEGIN( SvStruct ) ;
					}
<FindMembers>{B}*");"	{
						if(roundCount > 0){
							roundCount--;
						}
						if(current_root->insidePortList)
						{
							current_root->insidePortList = FALSE;
							unput(';');
						}
					}					
<SvStruct>{B}*"{"{B}*	{ 
            current->fileName = yyFileName ;
					  current->startLine = yyLineNr ;
					  current->name = removeRedundantWhiteSpace(current->name);
					  curlyCount=0;
					  BEGIN( ReadBody ) ;
					}
<SvStructName>{ID} {
					  current->name = yytext ;
					  lineCount();
					}
<SvStructName>";" {
					  current->endBodyLine = yyLineNr;
					  QCString &cn = current->name;
					  QCString rn = current_root->name.copy();
					  //printf("cn=`%s' rn=`%s'\n",cn.data(),rn.data());
					  if (!cn.isEmpty() && !rn.isEmpty())
					  {
					    prependScope();
					  }
					  // add compound definition to the tree
					  current->args=removeRedundantWhiteSpace(current->args);
  					current->type = current->type.simplifyWhiteSpace();
  					current->name = current->name.stripWhiteSpace();
					  //printf("adding `%s' `%s' `%s' brief=%s insideObjC=%d %x\n",current->type.data(),current->name.data(),current->args.data(),current->brief.data(),insideObjC,current->section);
					  current_root->addSubEntry( current ) ;
						current = new Entry(*current);
						current->reset();
						initEntry();
						BEGIN( FindMembers );
					}
<FindMembers>{B}*{TYPEDEFPREFIX}"enum"({BN}+("class"|"struct"))?{BN}+ {
					  isTypedef=((QCString)yytext).find("typedef")!=-1;
  					current->section = Entry::ENUM_SEC ;
					  addType( current ) ;
					  current->type += " enum" ;
					  current->fileName  = yyFileName;
					  current->startLine = yyLineNr;
					  current->bodyLine  = yyLineNr;
					  lineCount() ;
					  if (yytext[yyleng-1]=='{') unput('{');
					  current->name = yytext;
					  BEGIN( Enum ) ;
					}
<Enum>";"			{
					  isTypedef = FALSE;
					  BEGIN(FindMembers);
					}
<Enum>{ID}				{
					  current->name += yytext;
					  current->args += yytext;
					}
<Enum>"{"			{
					  BEGIN( ReadEnumBody );
					}
<Enum>"["			{
					  current->args += *yytext ;
					  squareCount++;
					}
<Enum>"]"				{ 
					  current->args += *yytext ;
					  squareCount--;
					}
<Enum>[^ \n\t\r]	{ 
					  current->args += *yytext ; 
					}
<ReadEnumBody>[^;\r\n\#{}"@'/]*	{
					  current->program += yytext ; 
					}
<ReadEnumBody>"}"	{
					  BEGIN (EnumName);
					}
<ReadEnumBody>";"			{
					  BEGIN(FindMembers);
					}
<EnumName>{ID}			{
					  current->systemVerilogType = SystemVerilogEnum;
					  oldName = current->name.copy();
					  oldProgram = current->program.data();
					  oldArgs = current->args.copy();
  					if (current->section == Entry::ENUM_SEC)
					  {
					    current->program+=","; // add field terminator
					  }
  				  current->name=yytext;
					  prependScope();
  					current->args = current->args.simplifyWhiteSpace();
  					current->type = current->type.simplifyWhiteSpace();
					  //printf("Adding compound %s %s %s\n",current->type.data(),current->name.data(),current->args.data());
					  current_root->addSubEntry( current ) ;
					  if (!firstTypedefEntry)
					  {
					    firstTypedefEntry = current;
					  }
					  current = new Entry;
					  initEntry();
					  isTypedef=TRUE; // to undo reset by initEntry()
  				}
<EnumName>","			{ 
					  current->section = Entry::ENUM_SEC ;
					  addType( current ) ;
					  current->type += " enum" ;
					  current->fileName  = yyFileName;
					  current->startLine = yyLineNr;
					  current->bodyLine  = yyLineNr;
					  lineCount() ;
					  //if (yytext[yyleng-1]=='{') unput('{');
					  current->name = oldName;
					  current->program = oldProgram;
					  current->args = oldArgs;
					}
<EnumName>";"		{
					  unput(*yytext);
					  BEGIN(Enum);
					}
<GetSignal,GetParameter>"="[^\n,;)({}\/]+			{
						skipSignal = TRUE;
						//printf("Found equal: %s\n", yytext);
						if(isParameter)
						{
							//printf("Parameter. Name: %s, value: %s\n", current->name, yytext+1);
							FileDef *fileDef;
							bool ambig;
							fileDef = findFileDef(Doxygen::inputNameDict,yyFileName,ambig);
							ParameterInfo *tmpParInfo = new ParameterInfo();
							tmpParInfo->fileName = fileDef->name();
							tmpParInfo->parameterName = current->name;
							parameterMap.insert(*tmpParInfo, * new QCString(yytext+1));
						}
					}
<GetSignal,GetParameter>"{"	{
						curlyCount++;
						lastCurlyContext = YY_START;
						BEGIN(SkipSignalCurly);
					}
<SkipSignalCurly>"}"	{
						curlyCount--;
						if(!curlyCount)
						{
							BEGIN(lastCurlyContext);
						}
					}
					
<GetParameter>"," {
					//addType( current ) ;
					//current->name = yytext;
					if(!insideParameterList)
					{
						unput('r');
						unput('e');
						unput('t');
						unput('e');
						unput('m');
						unput('a');
						unput('r');
						unput('a');
						unput('p');
						unput(' ');		
					}
					else
					{
						//addType( current ) ;
						//current->name = yytext;
					}
					unput(';');
					BEGIN(FindMembers);
				}
				
<GetSignal>","			{
						SystemVerilogType tmpSvType = current->systemVerilogType;
						if (current->bodyLine==-1)
						{
							current->bodyLine = yyLineNr;
						}
						bool needNewCurrent=FALSE;
						if (!current->name.isEmpty() && current->section!=Entry::ENUM_SEC)
						{
							current->type=current->type.simplifyWhiteSpace();
							current->args=removeRedundantWhiteSpace(current->args);
							current->name=current->name.stripWhiteSpace();
							current->section = Entry::VARIABLE_SEC ;
							current->fileName = yyFileName;
							current->startLine = yyLineNr;
							current_root->addSubEntry( current ) ;
							needNewCurrent=TRUE;
						}
						current = new Entry ;
						if (current->groups)
						{
							current->groups->clear();
						}
						current->protection = Private;
						current->name = svStatement;
						current->systemVerilogType = tmpSvType;
					}
<GetSignal>{ID}		{
						if(!skipSignal)
						{
							addType( current ) ;
							current->protection = Private;
							current->name = yytext;
						}
					}			
<GetParameter>"parameter" {
					  current->protection = Public;
					  current->name = yytext;
					  svStatement = yytext;
					  isParameter = TRUE;
					  current->systemVerilogType = SystemVerilogParameter;
					}
<GetParameter>{ID}		{
						lineCount();
						addType( current ) ;
						current->protection = Public;
						current->name = yytext;
					}
<GetParameter>")"	{
						roundCount--;
						if(roundCount==0)
						{
						insideParameterList = FALSE;
						current_root->insidePortList = TRUE;
						  unput(';');
							BEGIN(FindMembers);
						}
					}
<GetSignal>{ID}{B}*/";"	{
						if(!skipSignal)
						{
							addType( current ) ;
							current->name = yytext;
							BEGIN( FindMembers );
						}
						skipSignal = FALSE;
					}	
					
<GetSignal>{PACKAGESCOPE}		{
						if(!skipSignal)
						{
							addType( current ) ;
							current->protection = Private;
							current->name = yytext;
						}
					}
										
<GetSignal,GetParameter>";"			{
					  unput(*yytext);
					  isParameter = FALSE;
					  skipSignal = FALSE;
					  BEGIN( FindMembers ) ;
					}
<GetSignal,GetParameter>"("		{
						roundCount++;
					}
<GetSignal>")"	{
						roundCount--;
						if(roundCount<0)
						{
						  unput(';');
							BEGIN(FindMembers);
						}
					}
<GetSignal>"["[^ \n]+:[^ \n]+"]"	{
						if(!skipSignal)
						{
							addType( current ) ;
							current->name = yytext;
						}
					}
<GetSignal>"["[^ \n]+:[^ \n]+"]"{B}*/";"	{
					  if (!current->name.isEmpty() && !skipSignal)
					  {
					    current->args = yytext;
					    current->bodyLine = yyLineNr;
					    fullArgString=current->args.copy();
					    copyArgString=&current->args;
					    //printf(">>> Read function arguments current->argList->count()=%d\n",current->argList->count());
					  }
					  BEGIN( FindMembers ) ;
					}
<ModuleInstName>" "("#"{B}*"("[^)]*")")/{B}*{ID} {
					addType(current);
					current->name = yytext;
				}
<ModuleInstName>{ID} {
					  current->bodyLine = yyLineNr;
					  addType(current);
					  moduleCounter++;
					  //QCString tmpString;
					  //tmpString.setNum(moduleCounter);
					  current->name = yytext;
					  QCString tmpString = yytext;
					  int tmpInt = 0;
					  if(!moduleMap.contains(tmpString)) {
						moduleMap.insert(tmpString, 0);	
						tmpInt = 0;
					  }
					  else {
						tmpInt = moduleMap[tmpString];
						tmpInt++;
						moduleMap.replace(tmpString, tmpInt);
						tmpString.setNum(tmpInt);
						current->name += "_";
						current->name += tmpString;						
					  }
					}					
<ModuleInstName>"#"	{
					  BEGIN(ModuleNumberSign);
					}
<ModuleInstName>";"	{
					  current->systemVerilogType = SystemVerilogVariable;
					  unput(';');
					  BEGIN(lastContext);
					}
<ModuleInstName>","	{
						current->systemVerilogType = SystemVerilogVariable;
						if (current->bodyLine==-1)
						{
							current->bodyLine = yyLineNr;
						}
						bool needNewCurrent=FALSE;
						svStatement = current->type;
						if (!current->name.isEmpty() && current->section!=Entry::ENUM_SEC)
						{
							current->type=current->type.simplifyWhiteSpace();
							current->args=removeRedundantWhiteSpace(current->args);
							current->name=current->name.stripWhiteSpace();
							current->section = Entry::VARIABLE_SEC ;
							current->fileName = yyFileName;
							current->startLine = yyLineNr;
							current_root->addSubEntry( current ) ;
							needNewCurrent=TRUE;
						}
						current = new Entry ;
						if (current->groups)
						{
							current->groups->clear();
						}
						current->protection = Private;
						current->name = svStatement;
					}
<ModuleNumberSign>"("	{
					  roundCount++;
					}
<ModuleNumberSign>")"	{
					  roundCount--;
					  if(roundCount == 0)
					  {
							BEGIN(ModuleInstName);
						}
					}
<ModuleNumberSign>. {
					}
<ModuleInstName>"("			{
					  roundCount++;
						current_root->args += ' ';
						current_root->args += yytext;
						current_root->fileName = yyFileName ;
						current_root->startLine = yyLineNr ;
						current->systemVerilogType = SystemVerilogModuleInstantiation;
						if (!baseName.isEmpty())
	  					{
  							current_root->extends->append(new BaseInfo(baseName,baseProt,baseVirt));
						}
						baseName.resize(0);
					  BEGIN(ModuleInstPort);
					}
<ModuleInstPort>"."{ID}	{
					}	
<ModuleInstPort>"("		{
					  roundCount++;
					  BEGIN(ModuleInstNet);
					}	
<ModuleInstPort>")"		{
						roundCount--;
					  BEGIN(lastContext);
					}	
<ModuleInstNet>{ID}		{
					  BEGIN(ModuleInstNet);
					}
<ModuleInstNet>")"		{
					  roundCount--;
					  if(roundCount==1)
						{
							BEGIN(ModuleInstPort);
						}
					}
<ModuleInstNet>"("		{
					  roundCount++;
					}
<Property>{ID}/{B}*(";"|"(")			{
					  addType( current ) ;
					  if (current->bodyLine==-1)
					  {
						current->bodyLine = yyLineNr;
					  }
					  current->name = yytext;
					  
							current->type=current->type.simplifyWhiteSpace();
							current->args=removeRedundantWhiteSpace(current->args);
							current->name=current->name.stripWhiteSpace();
							current->section = Entry::VARIABLE_SEC ;
							current->fileName = yyFileName;
							current->startLine = yyLineNr;
							current_root->addSubEntry( current ) ;
							current = new Entry ;			  
					  
					  BEGIN(SkipProperty);
					}		
<SkipProperty>"endproperty"({B}*":"{B}*{ID})? {
						BEGIN(FindMembers);
					}			
<Always>"@"				{
					  addType( current ) ;
					  current->name = yytext;
					  hasDoneAlwaysParam = TRUE;
					  BEGIN(AlwaysParam);
					}
<AlwaysIf,AlwaysElse>"begin"			{
					  beginCount++;
						if( previous == NULL )
						{
							previous = current;
						}
					  lastCurlyContext = YY_START;
					  BEGIN(SkipCurly);
					}					
<Always>"begin"		{
					  beginCount++;
					  fakeName.setNum(yyLineNr);
					  fakeName = "always_noname"+fakeName;
					  int i;for (i=fakeName.length()-1;i>=0;i--)
					  {
					    unput(fakeName[i]);
					  }
					  BEGIN(AlwaysName);					
					}					
<Always>"begin"{BN}*":"			{
					  beginCount++;
					  BEGIN(AlwaysName);
					}
<Always>"end"{B}*":"{B}*{ID}			{
					  current->type.resize(0);
					  current->name.resize(0);
					  current->args.resize(0);
					  BEGIN(FindMembers);
					}
<Always>"always"|"always_ff"|"always_comb"|"always_latch"|"initial"		{
					  current->protection = Private;
						current->name = yytext;
					}
<Always>"end" {
					  current->type.resize(0);
					  current->name.resize(0);
					  current->args.resize(0);
						BEGIN(FindMembers);
					}
<Always,AlwaysIf,AlwaysElse>"if"{BN}*"(" {
						AlwaysInfo *tmpAlwaysInfo = new AlwaysInfo();
						tmpAlwaysInfo->alwaysContext = currentAlwaysContext;
						alwaysStack.push(tmpAlwaysInfo);
						currentAlwaysContext = "if";
						roundCount++;
						BEGIN(SkipAlwaysIf);
					}
<SkipAlwaysIf>")"	{
						roundCount--;
						if(!roundCount)
						{
							BEGIN(AlwaysIf);
						}
					}
<Always,AlwaysIf,AlwaysCase,AlwaysElse>"case" {
						AlwaysInfo *tmpAlwaysInfo = new AlwaysInfo();
						tmpAlwaysInfo->alwaysContext = currentAlwaysContext;
						alwaysStack.push(tmpAlwaysInfo);
						currentAlwaysContext = "case";
						BEGIN(AlwaysCase);
					}
<AlwaysMaybeElse>"else"	{
						currentAlwaysContext = "else";
						BEGIN(AlwaysElse);
					}
<AlwaysMaybeElse>[^ \n\r\t]	{
						unput(*yytext);
					  current->type.resize(0);
					  current->name.resize(0);
					  current->args.resize(0);
						BEGIN(FindMembers);
					}
<AlwaysIf>";"|"end" {
						BEGIN(AlwaysMaybeElse);						
					}
<AlwaysElse>";"|"end" {
						AlwaysInfo *tmpAlwaysInfo;
						bool tmpHasChanged = FALSE;
						while(!alwaysStack.isEmpty())
						{
							tmpAlwaysInfo = alwaysStack.pop();
							if(tmpAlwaysInfo->alwaysContext == "always")
							{
								currentAlwaysContext = tmpAlwaysInfo->alwaysContext;
								current->type.resize(0);
								current->name.resize(0);
								current->args.resize(0);
								BEGIN(FindMembers);
								tmpHasChanged = TRUE;
								break;
							}
							else if(tmpAlwaysInfo->alwaysContext == "if")
							{
								currentAlwaysContext = tmpAlwaysInfo->alwaysContext;
								BEGIN(AlwaysMaybeElse);
								tmpHasChanged = TRUE;
								break;
							}
						}
						if(!tmpHasChanged)
						{
							current->type.resize(0);
							current->name.resize(0);
							current->args.resize(0);
							BEGIN(FindMembers);
						}
					}
<AlwaysCase>"endcase"	{
						AlwaysInfo *tmpAlwaysInfo;
						bool tmpHasChanged = FALSE;
						while(!alwaysStack.isEmpty())
						{
							tmpAlwaysInfo = alwaysStack.pop();
							if(tmpAlwaysInfo->alwaysContext == "always")
							{
								currentAlwaysContext = tmpAlwaysInfo->alwaysContext;
								tmpHasChanged = TRUE;
								current->type.resize(0);
								current->name.resize(0);
								current->args.resize(0);
								BEGIN(FindMembers);
								break;
							}
							else if(tmpAlwaysInfo->alwaysContext == "if")
							{
								currentAlwaysContext = tmpAlwaysInfo->alwaysContext;
								tmpHasChanged = TRUE;
								BEGIN(AlwaysMaybeElse);
								break;
							}
							else if(tmpAlwaysInfo->alwaysContext == "case")
							{
								currentAlwaysContext = tmpAlwaysInfo->alwaysContext;
								tmpHasChanged = TRUE;
								BEGIN(AlwaysCase);
								break;	
							}
						}
						if(!tmpHasChanged)
						{
							current->type.resize(0);
							current->name.resize(0);
							current->args.resize(0);
							BEGIN(FindMembers);
						}
					}
<Always>";"		{
					  current->type.resize(0);
					  current->name.resize(0);
					  current->args.resize(0);
						BEGIN(FindMembers);
					}
<AlwaysName>{ID}			{
					  addType( current ) ;
					  printf("Always \"%s\" found...\n", yytext);
						current->name = yytext;
						current->bodyLine = yyLineNr;
						bool needNewCurrent=FALSE;
						if (!current->name.isEmpty() && current->section!=Entry::ENUM_SEC)
						{
						  current->type=current->type.simplifyWhiteSpace();
							current->args=removeRedundantWhiteSpace(current->args);
							current->name=current->name.stripWhiteSpace();
							current->section = Entry::VARIABLE_SEC ;
							current->fileName = yyFileName;
							current->startLine = yyLineNr;
							current_root->addSubEntry( current );
							needNewCurrent=TRUE;
						}
						if (needNewCurrent)
						{
							current = new Entry ;
						}
						else if (current->groups)
						{
							current->groups->clear();
						}
						initEntry();
						lastCurlyContext = Always;
						BEGIN( SkipCurly ) ;
					}
<AlwaysParam>{B}*"("{B}*	{
					  addType( current ) ;
					  current->name = yytext;
					  roundCount++;
					}
<AlwaysParam>{ID}	{
					  addType( current ) ;
					  current->name = yytext;
					}
<AlwaysParam>{ID}"."{ID}	{
					  addType( current ) ;
					  current->name = yytext;
					}
<AlwaysParam>{B}*")"{B}*	{
					  addType( current ) ;
					  current->name = yytext;					  
					  roundCount--;
					  BEGIN(Always);
					}
<FindMembers>"task"	{
					  lineCount();
					  addType( current ) ;
					  current->name  = yytext;
					  current->systemVerilogType = SystemVerilogTask;
					  BEGIN(SvFunction);
					}
<FindMembers>"function"	{
					  lineCount();
					  addType( current ) ;
					  current->name  = yytext;
					  current->systemVerilogType = SystemVerilogFunction;
					  BEGIN(SvFunction);
					}
<SvFunction>"(" {
            if (!current->name.isEmpty())
					  {
					    roundCount++;
					    current->args = yytext;
					    current->bodyLine = yyLineNr;
					    //currentArgumentContext = FuncQual;
					    fullArgString=current->args.copy();
					    copyArgString=&current->args;
					    BEGIN( SvFunctionArgs ) ;
					  }
					}
<SvFunction>"endfunction"|"endtask" {
						BEGIN(FindMembers);
					}
<SvFunction>{ID} {
						addType( current );
						current->name  = yytext;
					}
<SvFunctionArgs>[^ \/\r\t\n\)\(\"\'#]+ { 
					  *copyArgString+=yytext;
  					fullArgString+=yytext;
  				}
<SvFunctionArgs>"(" { 
						roundCount++;
						*copyArgString+=yytext;
  					fullArgString+=yytext;
  				}
<SvFunctionArgs>")" { 
						roundCount--;
						*copyArgString+=yytext;
  					fullArgString+=yytext;
						if(roundCount==0)
						{
							BEGIN(SvFunction);
						}
  				}
<SvFunctionArgs>{BN}*	{
  					*copyArgString+=" ";
  					fullArgString+=" ";
  					lineCount();
  				}
<SvFunction>";" {
					  current->name=current->name.simplifyWhiteSpace();
  					current->type=current->type.simplifyWhiteSpace();
					  current->args=removeRedundantWhiteSpace(current->args);
					  current->fileName = yyFileName;
					  current->startLine = yyLineNr;
					  if (current->bodyLine==-1)
					  {
					    current->bodyLine = yyLineNr;
					  }
					  static QRegExp re("([^)]*\\*[^)]*)"); // (...*...)
					  if (!current->type.isEmpty() && (current->type.find(re,0)!=-1 || current->type.left(8)=="typedef "))
					  {
					    current->section = Entry::VARIABLE_SEC;
					  }
					  else
					  {
					    current->section = Entry::FUNCTION_SEC;
					    current->proto = TRUE;
					  }
					  previous = current;
					  current_root->addSubEntry(current);
					  current = new Entry ;
					  initEntry();
					  BEGIN(SvFunctionBody);
					}
<SvFunctionBody>"endfunction"|"endtask" {
						BEGIN(FindMembers);
					}
<FindMembers>"modport"	{
					  lineCount();
					  addType( current ) ;
					  current->name  = yytext;
					  current->systemVerilogType = SystemVerilogModport;
					  insideModport = TRUE;
					  current_root->insidePortList = TRUE;
					}
<FindMembers>"parameter" {
					  current->name = yytext;
					  svStatement = yytext;
					  isParameter = TRUE;
					  current->protection = Public;
					  current->systemVerilogType = SystemVerilogParameter;
					  BEGIN( GetParameter );
					}
<FindMembers>"typedef" {
						current->protection = Private;
					  current->systemVerilogType = SystemVerilogTypedef;
					  current->name = yytext;
					  svStatement = yytext;
					  roundCount = 0;
					  skipSignal = FALSE;
					  BEGIN( GetSignal );
					}
<FindMembers>{CONSTTYPES} {
						current->protection = Private;
						current->name = yytext;
						svStatement = yytext;
						current->systemVerilogType = SystemVerilogConstant;
						BEGIN( GetSignal );
					}
<FindMembers>{SVTYPES} {
					  if(current_root->insidePortList)
					  {
							REJECT;
					  }
					  current->protection = Private;
					  current->systemVerilogType = SystemVerilogVariable;
					  current->name = yytext;
					  svStatement = yytext;
					  roundCount = 0;
					  skipSignal = FALSE;
					  BEGIN( GetSignal );
					}
<FindMembers>{PACKAGESCOPE} {
						if(current_root->insidePortList)
						{	
							REJECT;
						}		
						current->protection = Private;
						current->systemVerilogType = SystemVerilogVariable;
						current->name = yytext;
						svStatement = yytext;
						roundCount = 0;
						skipSignal = FALSE;
						BEGIN( GetSignal );										
					}		
<FindMembers>"import " {
						if(current_root->insidePortList)
						{	
							REJECT;
						}
						current->protection = Private;				
						current->systemVerilogType = SystemVerilogImport;
						current->name = yytext;
						svStatement = yytext;
						roundCount = 0;
						skipSignal = FALSE;
						BEGIN( GetSignal );						
					}						
<ReadPorts>"("			{
					  roundCount++;
					}
<ReadPorts>")"			{
					  roundCount--;
					}
<ReadPorts>";"			{
					  BEGIN(FindMembers);
					}
<ReadPorts>{ID}	{ 
					}
<Assign>";"				{
					  BEGIN(FindMembers);
					}
<Generate,GenerateElse,GenerateIf,GenerateDoFor,GenerateDoIf,GenerateDoElse,GenerateFor,GenerateSkip,GenerateSkipElse>"endgenerate"	{
					  BEGIN(FindMembers);
					}
<Generate,GenerateIf,GenerateDoFor,GenerateDoIf,GenerateDoElse,GenerateFor>"if"{BN}*"("	{
					  generateString = yytext;  
					  //printf("Setting generateContext to generateIf: %d\n", GenerateIf);
					  if(generateContext == 0)
					  {
							generateContext = GenerateIf;
					  }
					  else
					  {
							//printf("Pushing generateStack 1: %d\n", generateContext);
							int tmpContext = generateContext;
							generateStack.push(&tmpContext);
							
							generateContext = GenerateIf;
					  }
					  BEGIN(GenerateIf);
					}
<Generate,GenerateIf,GenerateDoFor,GenerateDoIf,GenerateDoElse,GenerateFor>"for"{BN}*/"("	{
					  generateString = yytext;
					  //printf("Setting generateContext to GenerateFor: %d\n", GenerateFor);
					  if(generateContext == 0)
					  {
							generateContext = GenerateFor;
					  }
					  else
					  {
							//printf("Pushing generateStack 2: %d\n", generateContext);
							int tmpContext = generateContext;
							generateStack.push(&tmpContext);							
							generateContext = GenerateFor;
					  }
					  BEGIN(GenerateFor);
					}
<GenerateIf>{ID}		{
					  FileDef	*fileDef;
					  bool ambig;
					  fileDef = findFileDef(Doxygen::inputNameDict,yyFileName,ambig);
					  ParameterInfo *tmpParInfo = new ParameterInfo();
					  QCString tmpParameter;
					  
						// Finding included files
						QList<IncludeInfo> *tmpIncludeList = fileDef->includeFileList();
						if(tmpIncludeList != NULL)
						{
							for(unsigned int i = 0; i < tmpIncludeList->count(); i++)
							{
								tmpParInfo->fileName = tmpIncludeList->at(i)->includeName;
								tmpParInfo->parameterName = yytext;
								tmpParameter = parameterMap[*tmpParInfo];
								if(tmpParameter != NULL)
								{
									generateString += tmpParameter;
									break;
								}
							}
						}
						if(tmpParameter == NULL)
						{					  		
							tmpParInfo->fileName = fileDef->name();
							tmpParInfo->parameterName = yytext;
							if(parameterMap[*tmpParInfo] != 0) {
								tmpParameter = parameterMap[*tmpParInfo];
								generateString += tmpParameter;
							}
							else
							{
			  					warn(yyFileName,yyLineNr, "Warning: could not find the value of parameter %s\n", yytext);
								BEGIN(GenerateSkip);							
							}
					  }
					}
<GenerateIf>{LOGICOP}		{
					  generateString += yytext;
					}
<GenerateIf>[0-9]+		{
					  generateString += yytext;
					}
<GenerateIf>")"			{
					  generateString += yytext;
					  //printf("Computing: '%s'\n", generateString);
					  bool tmpbool = computeGenerate(generateString);
					  if(tmpbool)
					  {
							generateContext = GenerateDoIf;
							BEGIN(GenerateDoIf);
					  }
					  else
					  {
							elseBeginCount = 0;
							generateContext = GenerateElse;
							//printf("Setting the generateContext to GenerateElse: %d\n", GenerateElse);
							if(!generateStack.isEmpty())
							{
								//printf("Popping the generateStack 5: %d\n", *generateStack.pop());
								
							}							
							BEGIN(GenerateElse);
					  }
					}
<GenerateIf>.			{
					  generateString += yytext;
					}

<GenerateFor>"("{B}*"genvar"*{B}*{ID}{B}*"="{B}*[^;]+";"[^\n]+";"[^\n]+/")"	{					
						QCString * tmpForString = new QCString(yytext);
						FileDef *fileDef;
						bool ambig;
						fileDef = findFileDef(Doxygen::inputNameDict,yyFileName,ambig);
						ParameterInfo *tmpParInfo = new ParameterInfo();
						tmpParInfo->fileName = fileDef->name();
						ForLoopInfo *tmpForLoopInfo = new ForLoopInfo();
						int tmpIndex = tmpForString->find('=',0,FALSE);
						bool loopIsValid = TRUE;
						if( tmpIndex != -1 )
						{
							int genvarIndex = 1;
							int tmpIndex2 = tmpForString->find("genvar ",0,FALSE);
							if( tmpIndex2 != -1) {
								genvarIndex = tmpIndex2 + 6;
							}
							tmpParInfo->parameterName = (tmpForString->mid(genvarIndex,tmpIndex-genvarIndex)).stripWhiteSpace();
							//printf("Variable name: %s\n", (tmpForString->mid(genvarIndex,tmpIndex-genvarIndex)).stripWhiteSpace());
							tmpForLoopInfo->variableName = (tmpForString->mid(genvarIndex,tmpIndex-genvarIndex)).stripWhiteSpace();
							int tmpColon1Index = tmpForString->find(';', tmpIndex, FALSE);
							if(tmpColon1Index != -1){
								parameterMap.insert(*tmpParInfo, (tmpForString->mid(tmpIndex + 1, tmpColon1Index - tmpIndex - 1)).stripWhiteSpace());
								int tmpColon2Index = tmpForString->find(';', tmpColon1Index + 1, FALSE);
								//printf("Constant expression: '%s'\n", tmpForString->mid(tmpColon1Index + 1, tmpColon2Index - tmpColon1Index - 1).stripWhiteSpace());
								tmpForLoopInfo->constantExpression = tmpForString->mid(tmpColon1Index + 1, tmpColon2Index - tmpColon1Index - 1).stripWhiteSpace();
								//printf("Genvar Assignment: '%s'\n", tmpForString->right(tmpForString->length() - tmpColon2Index - 1).stripWhiteSpace());
								if( (tmpForString->right(tmpForString->length() - tmpColon2Index - 1).stripWhiteSpace()).find("++",0,FALSE) != -1 )
								{
									tmpForLoopInfo->increment = TRUE;
								}
								else if( (tmpForString->right(tmpForString->length() - tmpColon2Index - 1).stripWhiteSpace()).find("--",0,FALSE) != -1 )
								{
									tmpForLoopInfo->increment = FALSE;
								}
								else
								{
									//printf("Error evaluating for-loop\n");
									loopIsValid = FALSE;
								}
								//printf("ParameterTing: '%s'\n", (tmpForString->mid(tmpIndex + 1, tmpColon1Index - tmpIndex - 1)).stripWhiteSpace());
							}
							else
							{
								loopIsValid = FALSE;
							}
						}
						if(loopIsValid) {
							//printf("Pushing the struct to the loop-stack\n");
							forLoopStack.push(tmpForLoopInfo);
							//beginCountFor = beginCount;
							beginCountFor = 0;
							BEGIN(GenerateForSave);
						}				
					}	
<GenerateForSave>"begin"(":"{B}*{ID})? {				
						beginCountFor++;	
						forLoopStack.top()->loopContent += yytext;
					}		
<GenerateForSave>. {
						forLoopStack.top()->loopContent += yytext;
					}	
<GenerateForSave>\n {
						forLoopStack.top()->loopContent += yytext;	
					}	
<GenerateForSave>("//"{B}*)?"/**"/[^/*] {
						forLoopStack.top()->loopContent += yytext;
					}																			
<GenerateForSave>"end"(":"{B}*{ID})? {
						forLoopStack.top()->loopContent += yytext;
						beginCountFor--;
						if(beginCountFor == 0){
							//printf("Unputting\n");
							int i = 0;
							for ( i = forLoopStack.top()->loopContent.length() - 1; i >= 0; --i ) {
								//printf("GenerateForSave 4. unputting: %c\n", forLoopStack.top()->loopContent.at(i).latin1());
								unput( forLoopStack.top()->loopContent.at(i).latin1());
							}
							//printf("The loop content: %s\n", forLoopStack.top()->loopContent.ascii());
							QCString tmpString = forLoopStack.top()->loopContent.ascii();
							int tmpInt = 0;
							tmpInt = tmpString.contains('\n', FALSE);				
							BEGIN(GenerateFor);
						}
					}
<GenerateElse,GenerateSkipElse>"begin"({BN}*:{BN}*{ID})? {
					  elseBeginCount++;
					  beginCount++;
					}

<GenerateSkipElse>"end"({BN}*:{BN}*{ID})? {
					  beginCount--;
					  elseBeginCount--;
					  if(elseBeginCount <= 0)
					  {
							if(!generateStack.isEmpty())
							{
								generateContext = *generateStack.pop();
								//printf("Popping the generateStack 1: %d\n", generateContext);
							}
							else
							{
								generateContext = 0;
							}
							if(generateContext==GenerateIf)
							{
								BEGIN(GenerateIf);
							}
							else if(generateContext==GenerateFor)
							{
								BEGIN(GenerateDoFor);
							}
							else if(generateContext==GenerateElse)
							{
								BEGIN(GenerateElse);
							}
							else if(generateContext==Generate)
							{
								BEGIN(Generate);
							}
							else if(generateContext!=0)
							{
								BEGIN(generateContext);
							}
							else
							{
								BEGIN(Generate);
							}
					  }  
					}			
<GenerateElse>"else"	{
					  if(elseBeginCount == 0)
					  {
							BEGIN(GenerateDoElse);
					  }
					}
<GenerateDoFor,GenerateDoIf,GenerateDoElse>"begin"({BN}*:{BN}*{ID})? {
					  beginCount++;
					}
<GenerateFor>"begin"({BN}*:{BN}*{ID})? {
					  beginCount++;
					  //printf("Setting generateContext to GenerateDoFor: %d\n", GenerateDoFor);
					  generateContext = GenerateDoFor;
					  BEGIN(GenerateDoFor);
					}
<GenerateFor,GenerateDoFor,GenerateDoIf,GenerateDoElse,GenerateIf>"end"({BN}*:{BN}*{ID})? {
					  beginCount--;
					  // Finding the end of the loop
					  if(beginCount == forLoopStack.count()-1){
						  if(!forLoopStack.isEmpty())
						  {
								// increment or decrement
								ParameterInfo *tmpParInfo = new ParameterInfo();
								FileDef *fileDef;
								bool ambig;
								fileDef = findFileDef(Doxygen::inputNameDict,yyFileName,ambig);							
								tmpParInfo->fileName = fileDef->name();
								tmpParInfo->parameterName = forLoopStack.top()->variableName;
								QCString tmpParameter;
								tmpParameter = parameterMap[*tmpParInfo];
								int tmpParValue = 0;
								if(tmpParameter != NULL)
								{													
									bool *isANumber = NULL;
									tmpParValue = tmpParameter.toInt(isANumber);
								}
								else {
									//printf("Error finding genvar\n");
								}
								if( forLoopStack.top()->increment) {
									tmpParValue++;
								}
								else{
									tmpParValue--;
								}
								tmpParameter.setNum(tmpParValue);
								//printf("Inserting value: '%s'\n", tmpParameter);
								//printf("parameterName: '%s'\n", tmpParInfo->parameterName);
								//printf("fileName: '%s'\n", tmpParInfo->fileName);
								parameterMap.replace(*tmpParInfo, tmpParameter);	

								QCString tmpToBeCalculated;
								QCString tmpStringValue;
								tmpStringValue.setNum(tmpParValue);
								int variableIndex;
								tmpToBeCalculated = forLoopStack.top()->constantExpression;
								variableIndex = tmpToBeCalculated.find(forLoopStack.top()->variableName);
								//printf("tmpStringValue: %s\n", tmpStringValue);
								//printf("variable index: %d\n", variableIndex);
								//printf("Variable length: %d\n", forLoopStack.top()->variableName.length());
								if(variableIndex != -1) {
									tmpToBeCalculated.replace(variableIndex, forLoopStack.top()->variableName.length(), tmpStringValue);
									//printf("tmpToBeCalculated: %s\n", tmpToBeCalculated);
									tmpToBeCalculated = replaceVariables(tmpToBeCalculated);
									//printf("Have replaced variables\n");
									tmpToBeCalculated.prepend("if(");
									tmpToBeCalculated.append(")");			
									bool tmpbool = computeGenerate(tmpToBeCalculated);
									//printf("FALSE: %d\n", FALSE);
									//printf("TRUE: %d\n", TRUE);
									//printf("The value have been calculated and found to be: %d\n", tmpbool);
									if(tmpbool) {
										//printf("The loop is not done\n");
										int i = 0;
										for ( i = forLoopStack.top()->loopContent.length() - 1; i >= 0; --i ) {
											//printf(" unputting: %c\n", forLoopStack.top()->loopContent.at(i).latin1());
											unput( forLoopStack.top()->loopContent.at(i).latin1());
										}
										//printf("The loop content: %s\n", forLoopStack.top()->loopContent.ascii());
										QCString tmpString = forLoopStack.top()->loopContent.ascii();
										int tmpInt = 0;
										tmpInt = tmpString.contains('\n', FALSE);
										//printf("Number of lines: %d\n", tmpInt);	
										yyLineNr = yyLineNr - tmpInt;												
										generateContext = GenerateDoFor;
										//printf("Setting GenerateContext to GenerateDoFor: %d\n", generateContext);
										BEGIN(GenerateDoFor);	
									}
									else{
										//printf("For-loop done\n");
										forLoopStack.pop();
										if(generateContext == GenerateDoIf)
										{
											BEGIN(GenerateSkipElse);					  
										}
										else if(!generateStack.isEmpty())
										{
											generateContext = *generateStack.pop();
											//printf("Popping the generateStack 2: %d\n", generateContext);
											if(generateContext == GenerateDoIf)
											{
												BEGIN(GenerateDoIf);
											}
											else if(generateContext!=0)
											{
												BEGIN(generateContext);
											}
											else
											{
												BEGIN(Generate);
											}
										}
										else
										{
											generateContext = Generate;
											BEGIN(Generate);
										}									
									}
								}
								else
								{
									//printf("ERROR finding variable in constant expression\n");
								}
								BEGIN(GenerateFor);					  
						  }	
					  }			  
					  if(generateContext == GenerateDoIf)
					  {
							BEGIN(GenerateSkipElse);					  
					  }
					  else if(generateContext == GenerateDoFor)
					  {
							BEGIN(GenerateDoFor);
					  }
					  else if(!generateStack.isEmpty())
					  {
							generateContext = *generateStack.pop();
							//printf("Popping the generateStack 3: %d\n", generateContext);
							if(generateContext == GenerateDoIf)
							{
								BEGIN(GenerateDoIf);
							}
							else if(generateContext!=0)
							{
								//printf("Beginning generateContext\n");
								BEGIN(generateContext);
							}
							else
							{
								BEGIN(Generate);
							}
					  }
					  else
					  {
							generateContext = Generate;
							BEGIN(Generate);
					  }
					}
<GenerateElse>"end"({BN}*:{BN}*{ID})? {
					  beginCount--;
					  elseBeginCount--;
					  if(elseBeginCount < 0)
					  {
							if(!generateStack.isEmpty())
							{
								//printf("Popping the generateStack 4\n");
								generateContext = *generateStack.pop();
							}
							else
							{
								generateContext = 0;
							}
							if(generateContext==GenerateIf)
							{
								BEGIN(GenerateIf);
							}
							else if(generateContext==GenerateFor)
							{
								BEGIN(GenerateFor);
							}
							else if(generateContext==GenerateElse)
							{
								BEGIN(GenerateElse);
							}
							else if(generateContext!=0)
							{
								BEGIN(generateContext);
							}
							else
							{
								BEGIN(Generate);
							}
					  }  
					}					
<GenerateDoFor,GenerateDoIf,GenerateDoElse,Generate>"genvar"			{
					  current->protection = Private;
					  current->systemVerilogType = SystemVerilogVariable;
					  current->name = yytext;
					  BEGIN( GetGenerateSignal );
					}
<GetGenerateSignal>{ID}		{
						addType( current ) ;
						current->protection = Private;
						current->name = yytext;
					}	
<GetGenerateSignal>";"	{
						unput(*yytext);
						BEGIN(generateContext);
					}
<GenerateDoFor,GenerateDoIf,GenerateDoElse,Generate>{ID}			{
					  baseProt=Public;
					  baseVirt=Normal;
					  baseName.resize(0);	
					  current->name = yytext;									
					  baseName+=yytext;
						lastContext = YY_START;
						BEGIN( ModuleInstName );
					}
<GenerateDoFor,GenerateDoIf,GenerateDoElse,Generate>";"			{ 
					  if (current->bodyLine==-1)
					  {
					    current->bodyLine = yyLineNr;
					  }
					  if (isTypedef && current->type.left(8)!="typedef ")
					  {
					    current->type.prepend("typedef ");
					  }
					  bool needNewCurrent=FALSE;
					  if (!current->name.isEmpty() && current->section!=Entry::ENUM_SEC)
					  {
					    //printf("Adding subEntry. SystemVerilogType: %d\n", current->systemVerilogType);
					    current->type=current->type.simplifyWhiteSpace();
					    current->args=removeRedundantWhiteSpace(current->args);
					    current->name=current->name.stripWhiteSpace();
					    current->section = Entry::VARIABLE_SEC ;
					    current->fileName = yyFileName;
					    current->startLine = yyLineNr;
					    current_root->addSubEntry( current ) ;
					    needNewCurrent=TRUE;
					  }
					  current_root->insidePortList = FALSE;  
					  if (needNewCurrent)
					  {
					    current = new Entry ;
					  }
					  else if (current->groups)
					  {
					    current->groups->clear();
					  }
					  initEntry();		  
					}
<FindMembers>"begin"			{
						beginCount++;
						BEGIN(SkipCurly);
					}
<FindMembers>"#("	{
						lineCount();
						insideParameterList = TRUE;
						roundCount++;
						current->protection = Public;
						//addType( current ) ;
						//current_root->insidePortList = TRUE;
						//BEGIN(GetParameter);
					}
<FindMembers>{ID}/({BN}*":"{BN}*"assert") {
						lineCount();
						current->protection = Private;
						current->name = "assert";
						current->systemVerilogType = SystemVerilogAssertion;								
						addType( current ) ;
						if (current->bodyLine==-1)
						{	
							current->bodyLine = yyLineNr;
						}
						current->name = yytext;
						current->type=current->type.simplifyWhiteSpace();
						current->args=removeRedundantWhiteSpace(current->args);
						current->name=current->name.stripWhiteSpace();
						current->section = Entry::VARIABLE_SEC ;
						current->fileName = yyFileName;
						current->startLine = yyLineNr;
						current_root->addSubEntry( current ) ;
						current = new Entry ;			  
						BEGIN(SkipAssert);
					}
<SkipAssert>"begin" {
						beginCount++;
					}		
<SkipAssert>"end" {
						beginCount--;
						if(beginCount==0) 
						{
							BEGIN(FindMembers);
						}
					}
<CoverGroup>{ID}/";" {
					covergroupName = yytext;
					addType( current ) ;
					current->name = covergroupName;
					current->bodyLine = yyLineNr;
					bool needNewCurrent=FALSE;
					if (!current->name.isEmpty() && current->section!=Entry::ENUM_SEC)
					{
						current->type=current->type.simplifyWhiteSpace();
						current->args=removeRedundantWhiteSpace(current->args);
						current->name=current->name.stripWhiteSpace();
						current->section = Entry::VARIABLE_SEC ;
						current->fileName = yyFileName;
						current->startLine = yyLineNr;
						current_root->addSubEntry( current );
						needNewCurrent=TRUE;
					}
					if (needNewCurrent)
					{
						current = new Entry ;
					}
					else if (current->groups)
					{
						current->groups->clear();
					}
					initEntry();					
					BEGIN(CoverGroupSkip);	
				 }	
<CoverGroup>{ID} {
					covergroupName = yytext;
					BEGIN(CoverGroupEvent);
					}
<CoverGroupEvent>"@"[^\n\r]+/";" {
					addType( current ) ;
					current->name = yytext;

					addType( current ) ;
					current->name = covergroupName;
					current->bodyLine = yyLineNr;
					bool needNewCurrent=FALSE;
					if (!current->name.isEmpty() && current->section!=Entry::ENUM_SEC)
					{
						current->type=current->type.simplifyWhiteSpace();
						current->args=removeRedundantWhiteSpace(current->args);
						current->name=current->name.stripWhiteSpace();
						current->section = Entry::VARIABLE_SEC ;
						current->fileName = yyFileName;
						current->startLine = yyLineNr;
						current_root->addSubEntry( current );
						needNewCurrent=TRUE;
					}
					if (needNewCurrent)
					{
						current = new Entry ;
					}
					else if (current->groups)
					{
						current->groups->clear();
					}
					initEntry();					
				}
<CoverGroupEvent,CoverGroupSkip>"endgroup" {
					BEGIN(FindMembers);						
				}
<FindMembers>{ID}	{
					  lineCount();
					  addType( current ) ;
					  if (current_root->insidePortList && strcmp(yytext,"input")==0)
					  {
							current->protection = Public;
							current->name = yytext;
							current->systemVerilogType = SystemVerilogPort;
					  }
					  else if (current_root->insidePortList && strcmp(yytext,"output")==0)
					  {
							current->protection = Public;
					    current->name = yytext;
							current->systemVerilogType = SystemVerilogPort;
					  }
					  else if (current_root->insidePortList && strcmp(yytext,"inout")==0)
					  {
							current->protection = Public;
					    current->name = yytext;
							current->systemVerilogType = SystemVerilogPort;
					  }
					  else if(current_root->insidePortList)
					  {
							if(current->systemVerilogType == SystemVerilogNone)
							{
							  current->systemVerilogType = SystemVerilogPort;
							}
							current->protection = Public;
							current->name = yytext;
					  }
					  else
					  {
							if(strcmp(yytext,"always")==0 || strcmp(yytext,"always_ff")==0 || strcmp(yytext,"always_latch")==0 || strcmp(yytext,"always_comb")==0)
							{
								current->protection = Private;
								current->name = yytext;
								current->systemVerilogType = SystemVerilogAlwaysBlock;
								currentAlwaysContext = "always";
								alwaysStack.clear();
								BEGIN(Always);
							}
							else if(strcmp(yytext,"initial")==0)
							{
								current->protection = Private;
								current->name = yytext;
								current->systemVerilogType = SystemVerilogInitialBlock;
								currentAlwaysContext = "always";
								alwaysStack.clear();
								BEGIN(Always);
							}
							else if(strcmp(yytext,"assign")==0)
							{
								current->protection = Private;
								lineCount();
								current->startLine = yyLineNr;
								current->bodyLine  = yyLineNr;								
								current->name = yytext;
								addType(current);
								fakeName.setNum(yyLineNr);
								fakeName = "assign"+fakeName;
								current->name = fakeName;
								current->systemVerilogType = SystemVerilogAssign;
								BEGIN(Assign);
							}
							else if(strcmp(yytext,"covergroup")==0)
							{
								current->protection = Private;
								current->name = yytext;
								current->systemVerilogType = SystemVerilogCovergroup;							
								BEGIN(CoverGroup);
							}
							else if(strcmp(yytext,"generate")==0)
							{
								generateContext = Generate;
								//printf("Setting GenerateContext to Generate: %d\n", generateContext);
								BEGIN(Generate);
							}
							else if(strcmp(yytext,"property")==0)
							{
								current->protection = Private;
								current->name = yytext;
								current->systemVerilogType = SystemVerilogProperty;							
								BEGIN(Property);
							}
							else if(current->systemVerilogType == SystemVerilogTypedef)
							{
								current->protection = Public;
								current->name = yytext;
							}
							else
							{
								roundCount = 0;
								baseProt=Public;
								baseVirt=Normal;
								baseName.resize(0);
								current->name = yytext;
								baseName+=yytext;
								lastContext = YY_START;
								BEGIN( ModuleInstName );
							}
	    			}
					}
<FindMembers>"."			{
  				}
<FindMembers>"::"			{
					  current->name+=yytext;
  				}
  				/*
<FindMembers>{B}*"`"			{ 
					  lastCPPContext = YY_START;
					  BEGIN( SkipCPP ) ;
					}
					*/
<FindMembers>"`"{B}*"include"	{ 
						if(current_root->insidePortList)
						{	
							REJECT;
						}
						current->protection = Private;				
						current->systemVerilogType = SystemVerilogInclude;
						current->name = yytext;
						svStatement = yytext;
						roundCount = 0;
						skipSignal = FALSE;
						BEGIN( GetInclude );								  
					}	
<GetInclude>{ID}"."{ID} {
						addType( current ) ;
						current->name = yytext;
						unput(';');
						BEGIN( FindMembers );
					}								
<FindMembers>{B}*"`"{B}*"define"	{ 
  					current->bodyLine = yyLineNr;
  					current->systemVerilogType = SystemVerilogNone;
  					BEGIN( Define );
  				}
<FindMembers,ReadBody,SkipCurly>{B}*"#"{B}+[0-9]+{B}+/"\""	{ /* line control directive */
					  yyLineNr = atoi(&yytext[1]);
					  lastPreLineCtrlContext = YY_START;
  					BEGIN( PreLineCtrl );
  				}
<PreLineCtrl>"\""[^\n\"]*"\""		{
				    yyFileName = stripQuotes(yytext);
  				}
<PreLineCtrl>.				{
					}
<PreLineCtrl>\n				{
            yyLineNr++;
            current->program += yytext ; 
  					BEGIN( lastPreLineCtrlContext );
  				}
<SkipCPP>.				
<SkipCPP>\\[\r]*"\n"[\r]*		{ 
					  yyLineNr++ ; 
					}
<SkipCPP>[\r]*\n[\r]*			{ 
					  yyLineNr++ ;
					  BEGIN( lastCPPContext) ;
					}
<Define>{ID}{B}*"("			{
  					current->name = yytext;
					  current->name = current->name.left(current->name.length()-1).stripWhiteSpace();
					  current->args = "(";
  					current->bodyLine = yyLineNr;
					  currentArgumentContext = DefineEnd;
					  fullArgString=current->args.copy();
					  copyArgString=&current->args;
					  BEGIN( ReadFuncArgType ) ;
  				}
<Define>{ID}				{
  					//printf("Define `%s' without args\n",yytext);
  					current->bodyLine = yyLineNr;
  					current->name = yytext;
					  BEGIN(DefineEnd);
  				}
<DefineEnd>\n				{
					  yyLineNr++;
					  current->fileName   = yyFileName;
					  current->startLine  = yyLineNr;
					  current->type.resize(0);
  					current->args       = current->args.simplifyWhiteSpace();
  					current->name       = current->name.stripWhiteSpace();
					  current->section    = Entry::DEFINE_SEC;
					  current_root->addSubEntry(current);
					  current             = new Entry ;
					  initEntry();
					  BEGIN(FindMembers);
  				}
<DefineEnd>\\[\r]?\n			{
  					yyLineNr++;
  				}
<DefineEnd>\"				{
					  lastStringContext=DefineEnd;
					  BEGIN(SkipString);
  				}
<DefineEnd>.				

<FindMembers>[*&]+			{ 
            current->name += yytext ;  
						addType( current );
					}
<FindMembers,MemberSpec,Function,ReadInitializer>";"{BN}*("/**"|"//!"|"/*!"|"///")"<" {
  					lineCount();
					  if (current->bodyLine==-1)
					  {
					    //current->bodyLine=yyLineNr;
					  }
					  docBlockContext   = YY_START;
					  docBlockInBody    = FALSE;
					  docBlockAutoBrief = ( yytext[yyleng-2]=='*' && Config_getBool("JAVADOC_AUTOBRIEF") ) || ( yytext[yyleng-2]=='!' && Config_getBool("QT_AUTOBRIEF") );
					  docBlock.resize(0);
					  docBlockTerm = ';';
					  if (yytext[yyleng-3]=='/')
					  {
					    startCommentBlock(TRUE);
					    BEGIN( DocLine );
					  }
					  else
					  {
					    startCommentBlock(FALSE);
					    BEGIN( DocBlock );
					  }
  				}
<FindMembers,GetParameter>{BN}*("/**"|"//!"|"/*!"|"///")"<" |
<MemberSpec,FindFields,FindMembers,ReadInitializer>","{BN}*("/**"|"//!"|"/*!"|"///")"<" {
					  //printf("Found comment: %s\n", yytext);
  					lineCount();
					  docBlockContext   = YY_START;
					  docBlockInBody    = FALSE;
					  docBlockAutoBrief = ( yytext[yyleng-2]=='*' && Config_getBool("JAVADOC_AUTOBRIEF") ) || ( yytext[yyleng-2]=='!' && Config_getBool("QT_AUTOBRIEF") );
					  docBlock.resize(0);
					  docBlockTerm = ',';
					  if (yytext[yyleng-3]=='/')
					  {
					    startCommentBlock(TRUE);
					    BEGIN( DocLine );
					  }
					  else
					  {
					    startCommentBlock(FALSE);
					    BEGIN( DocBlock );
					  }
  				}
<DefineEnd,FindFields,ReadInitializer>{BN}*("/**"|"//!"|"/*!"|"///")"<" {
  					lineCount();
					  if (current->bodyLine==-1)
					  {
					    current->bodyLine=yyLineNr;
					  }
					  docBlockContext   = YY_START;
					  docBlockInBody    = FALSE;
					  docBlockAutoBrief = ( yytext[yyleng-2]=='*' && Config_getBool("JAVADOC_AUTOBRIEF") ) || ( yytext[yyleng-2]=='!' && Config_getBool("QT_AUTOBRIEF") );
					  docBlock.resize(0);
					  docBlockTerm = 0;
					  if (yytext[yyleng-3]=='/')
					  {
					    startCommentBlock(TRUE);
					    BEGIN( DocLine );
					  }
					  else
					  {
					    startCommentBlock(FALSE);
					    BEGIN( DocBlock );
					  }
  				}
<FindMembers,FindFields>("//"([!/]?){B}*{CMD}"{")|("/*"([!*]?){B}*{CMD}"{")	{
            if (previous && previous->section==Entry::GROUPDOC_SEC)
					  {
					    // link open command to the group defined in the previous entry
					    openGroup(previous,yyFileName,yyLineNr);
					  }
					  else
					  {
					    // link open command to the current entry
					    openGroup(current,yyFileName,yyLineNr);
					  }
					  initEntry();
					  if (yytext[1]=='/')
					  {
					    if (yytext[2]=='!' || yytext[2]=='/')
					    {
					      docBlockContext   = YY_START;
					      docBlockInBody    = FALSE;
					      docBlockAutoBrief = FALSE;
					      docBlock.resize(0);
					      docBlockTerm = 0;
					      startCommentBlock(TRUE);
					      BEGIN(DocLine);
					    }
					    else
					    {
					      lastCContext=YY_START;
					      BEGIN(SkipCxxComment);
					    }
					  }
					  else
					  {
					    if (yytext[2]=='!' || yytext[2]=='*')
					    {
					      docBlockContext   = YY_START;
					      docBlockInBody    = FALSE;
					      docBlock.resize(0);
					      docBlockAutoBrief = ( yytext[yyleng-2]=='*' && Config_getBool("JAVADOC_AUTOBRIEF") ) || ( yytext[yyleng-2]=='!' && Config_getBool("QT_AUTOBRIEF") );
					      docBlockTerm = 0;
					      startCommentBlock(FALSE);
					      BEGIN(DocBlock);
					    }
					    else
					    {
					      lastCContext=YY_START;
					      BEGIN(SkipComment);
					    }
					  }
  				}
<FindMembers,FindFields,ReadInitializer>"//"([!/]?){B}*{CMD}"}".*|"/*"([!*]?){B}*{CMD}"}".*"*/"	{
  					closeGroup(current,yyFileName,yyLineNr);
  				}
<FindMembers>"="			{ // in PHP code this could also be due to "<?="
  					current->bodyLine = yyLineNr;
					  lastInitializerContext = YY_START;
					  initBracketCount=0;
					  BEGIN(ReadInitializer);
  				}
  /* Read initializer rules */
<ReadInitializer>"("			{
  					lastRoundContext=YY_START;
  					pCopyRoundGString=&current->initializer;
					  roundCount=0;
  					current->initializer+=*yytext; 
  					BEGIN(GCopyRound);
  				}
<ReadInitializer>"{"			{
  					lastCurlyContext=YY_START;
  					pCopyCurlyGString=&current->initializer;
					  curlyCount=0;
  					current->initializer+=*yytext; 
  					BEGIN(GCopyCurly);
  				}
<ReadInitializer>[;,]			{
  					//printf(">> initializer `%s' <<\n",current->initializer.data());
  					if (*yytext==';' || lastInitializerContext==FindFields)
					  {
  						unput(*yytext);
  						BEGIN(lastInitializerContext);
					  }
					  else if (*yytext==',' && initBracketCount==0) // for "int a=0,b=0"
					  {
  						unput(*yytext);
  						BEGIN(lastInitializerContext);
					  }
					  else
					  {
  						current->initializer+=*yytext; 
					  }
  				}
<ReadInitializer>\"			{
            lastStringContext=YY_START;
  					current->initializer+=*yytext; 
  					pCopyQuotedGString=&current->initializer;
					  BEGIN(CopyGString);
					}
<ReadInitializer>"->"			{
  					current->initializer+=yytext; 
					}
<ReadInitializer>"<<"			{
  					current->initializer+=yytext; 
					}
<ReadInitializer>">>"			{
  					current->initializer+=yytext; 
  				}
<ReadInitializer>[<\[{(]		{
  					initBracketCount++;
  					current->initializer+=*yytext; 
  				}
<ReadInitializer>[>\]})]		{
  					initBracketCount--;
  					current->initializer+=*yytext; 
  				}
<ReadInitializer>\'			{	
					  current->initializer+=yytext; 
  				}
<ReadInitializer>{CHARLIT}              { 
					  current->initializer+=yytext; 
          } 
<ReadInitializer>\n			{
  					current->initializer+=*yytext;
					  yyLineNr++;
  				}
<SkipVerbString>[^\n"]+			{
					  *pSkipVerbString+=yytext;
					}
<SkipVerbString>"\"\""			{ // quote escape
					  *pSkipVerbString+=yytext;
					}
<SkipVerbString>"\""			{
					  *pSkipVerbString+=*yytext;
					  BEGIN(lastSkipVerbStringContext);
					}
<SkipVerbString>\n			{
					  *pSkipVerbString+=*yytext;
					  yyLineNr++;
  				}
<SkipVerbString>.			{
					  *pSkipVerbString+=*yytext;
  				}
<ReadInitializer>"?>"			{
					  current->initializer+=yytext;
  				}
<ReadInitializer>.			{ 
  					current->initializer+=*yytext; 
					}
  /* generic quoted growable string copy rules */
<CopyGString>\\.		{
  					*pCopyQuotedGString+=yytext;
  				}
<CopyGString>\"				{ 
  					*pCopyQuotedGString+=*yytext;
  					BEGIN( lastStringContext ); 
					}
<CopyGString>"/*"|"*/"|"//" {
  					*pCopyQuotedGString+=yytext;
  				}
<CopyGString>\n		{
  					*pCopyQuotedGString+=*yytext;
  					yyLineNr++;
  				}
<CopyGString>.		{
  					*pCopyQuotedGString+=*yytext;
  				}
  /* generic round bracket list copy rules for growable strings */
<GCopyRound>\"				{
					  *pCopyRoundGString+=*yytext;
  					pCopyQuotedGString=pCopyRoundGString;
					  lastStringContext=YY_START;
					  BEGIN(CopyGString);
					}
<GCopyRound>"("				{
  					*pCopyRoundGString+=*yytext;
  					roundCount++;
  				}
<GCopyRound>")"				{
  					*pCopyRoundGString+=*yytext;
					  if (--roundCount<0)
					  BEGIN(lastRoundContext);
  				}
<GCopyRound>\n				{
  					yyLineNr++;
  					*pCopyRoundGString+=*yytext;
  				}
<GCopyRound>\'				{
					  *pCopyRoundGString+=yytext;
  				}
<GCopyRound>{CHARLIT}		{ 
            *pCopyRoundGString+=yytext; 
          }
<GCopyRound>[^"'()\n]+			{
  					*pCopyRoundGString+=yytext;
  				}
<GCopyRound>.				{
  					*pCopyRoundGString+=*yytext;
  				}
  /* generic curly bracket list copy rules for growable strings */
<GCopyCurly>\"				{
					  *pCopyCurlyGString+=*yytext;
  					pCopyQuotedGString=pCopyCurlyGString;
					  lastStringContext=YY_START;
					  BEGIN(CopyGString);
					}
<GCopyCurly>\'				{
					  *pCopyCurlyGString+=*yytext;
					}
<GCopyCurly>"{"				{
  					*pCopyCurlyGString+=*yytext;
					  curlyCount++;
  				}
<GCopyCurly>"}"				{
					  *pCopyCurlyGString+=*yytext;
					  if (--curlyCount<0)
					    BEGIN(lastCurlyContext); 
  				}
<GCopyCurly>{CHARLIT}       { 
					  *pCopyCurlyGString+=yytext; 
					}
<GCopyCurly>[^"'{}\/\n]+			{
  					*pCopyCurlyGString+=yytext;
  				}
<GCopyCurly>"/"				{ 
						*pCopyCurlyGString+=yytext; 
					}
<GCopyCurly>\n				{
  					yyLineNr++;
					  *pCopyCurlyGString+=*yytext;
  				}
<GCopyCurly>.				{
					  *pCopyCurlyGString+=*yytext;
  				}

  /* ---------------------- */


<FindMembers>":"			{
					}
<FindMembers>[;,]			{ 
					  if (current->bodyLine==-1)
					  {
					    current->bodyLine = yyLineNr;
					  }
					  if (isTypedef && current->type.left(8)!="typedef ")
					  {
					    current->type.prepend("typedef ");
					  }
					  bool needNewCurrent=FALSE;
					  if (!current->name.isEmpty() && current->section!=Entry::ENUM_SEC)
					  {
					    //printf("Adding subEntry. name: '%s', type: '%s', args: '%s'\n", current->name, current->type, current->args);
					    current->type=current->type.simplifyWhiteSpace();
					    current->args=removeRedundantWhiteSpace(current->args);
					    current->name=current->name.stripWhiteSpace();
					    current->section = Entry::VARIABLE_SEC ;
					    current->fileName = yyFileName;
					    current->startLine = yyLineNr;
					    current_root->addSubEntry( current ) ;
					    needNewCurrent=TRUE;
					  }
					  if(*yytext == ';' && insideParameterList)
					  {
							//insideParameterList = FALSE;
					    if (needNewCurrent)
					    {
					      current = new Entry ;
					    }
					    else if (current->groups)
					    {
					      current->groups->clear();
					    }
					    initEntry();
					  }
					  else if(*yytext == ';')
					  {
							//current_root->insidePortList = FALSE;
					    if (needNewCurrent)
					    {
					      current = new Entry ;
					    }
					    else if (current->groups)
					    {
					      current->groups->clear();
					    }
					    initEntry();
					  }
					  else
					  {
					    if (needNewCurrent)
					    {
					      current = new Entry ;
					    }
					    else if (current->groups)
					    {
					      current->groups->clear();
					    }
					    initEntry();
					  }
					  BEGIN( FindMembers ) ;
					}
<FindMembers>"["			{ 
  					current->args += yytext ;
					  squareCount=1;
					  BEGIN( Array ) ;
					}
<Array>"]"				{
					  current->args += *yytext ;
					  if (--squareCount<=0)
							BEGIN( FindMembers ) ;
					}
<Array>"["				{
					  current->args += *yytext ;
					  squareCount++;	
					}
<Array>.				{
					  current->args += *yytext ; 
					}
<FindFields>{ID}			{
  					current->bodyLine = yyLineNr;
  					current->name     = yytext;
					}
<FindFields>"("				{
  					// Java enum initializer
  					unput('(');
  					lastInitializerContext = YY_START;
					  initBracketCount=0;
  					BEGIN(ReadInitializer);
  				}
<FindFields>"="				{
  					lastInitializerContext = YY_START;
					  initBracketCount=0;
  					BEGIN(ReadInitializer);
  				}
<FindFields>","				{
					  //printf("adding `%s' `%s' `%s' to enum `%s' (mGrpId=%d)\n",
					  //     current->type.data(), current->name.data(),
					  //     current->args.data(), current_root->name.data(),current->mGrpId);
  					if (!current->name.isEmpty())
					  {
					    current->fileName   = yyFileName;
					    current->startLine  = yyLineNr;
					    current->type       = "@"; // enum marker
  					  current->args       = current->args.simplifyWhiteSpace();
  					  current->name       = current->name.stripWhiteSpace();
					    current->section    = Entry::VARIABLE_SEC;
					    // add to the scope of the enum
					    current_root->addSubEntry(current);
					    current             = new Entry(*current);
					    // add to the scope surrounding the enum (copy!)
					    current_root->parent()->addSubEntry(current);
					    current             = new Entry ;
					    initEntry();
					  }
					  else // probably a redundant , 
					  {
							current->reset();
					  }
  				}
<ReadBody>{B}*("begin"{BN})|("begin"{BN}*":")	{
					  ++curlyCount;
					  current->program += yytext ;
					}
<ReadBody>{B}*("end"({BN}))|("end"{BN}*":")	{
					  --curlyCount ;
					  current->program += yytext ;
					}
<ReadBody>"}"|"endmodule"|"endinterface"|"endclass"|"endpackage"	{ //err("ReadBody count=%d\n",curlyCount);
  					if ( curlyCount>0 )
					  {
					    current->program += yytext ;
					    --curlyCount ;
					  }
					  else if(current->spec == Entry::Struct)
					  {
							BEGIN(SvStructName);
					  }
					  else
					  {
					    current->endBodyLine = yyLineNr;
					    QCString &cn = current->name;
					    QCString rn = current_root->name.copy();
					    //printf("cn=`%s' rn=`%s'\n",cn.data(),rn.data());
					    if (!cn.isEmpty() && !rn.isEmpty())
					    {
					      prependScope();
					    }
					    if (isTypedef && cn.isEmpty())
					    {
					      BEGIN( TypedefName );
					    }
					    else
					    {
					      if (current->section == Entry::ENUM_SEC)
					      {
					        current->program+=','; // add field terminator
					      }
					      // add compound definition to the tree
					      current->args=removeRedundantWhiteSpace(current->args);
  					    current->type = current->type.simplifyWhiteSpace();
  					    current->name = current->name.stripWhiteSpace();
					      //printf("adding `%s' `%s' `%s' brief=%s %x\n",current->type.data(),current->name.data(),current->args.data(),current->brief.data(),current->section);
					      current_root->addSubEntry( current ) ;
					      bool tmpBool = isTypedef;
								current = new Entry(*current);
								current->reset();
								initEntry();
								if(tmpBool)
								{
									current->systemVerilogType = SystemVerilogTypedef;
								}
								BEGIN( FindMembers );
					    }
					  }
					}
<ReadBody>[^\r\n\#{}"@'/ ]*	{
					  current->program += yytext ; 
					}
<ReadBody,ReadPorts>"//".*		{ 
					  current->program += yytext ; 
					}
<ReadBody>@\"	{ 
						current->program += yytext ; 
						pSkipVerbString = &current->program;
						lastSkipVerbStringContext=YY_START;
            BEGIN( SkipVerbString );
					}
<ReadBody>\"	{ 
						current->program += yytext ; 
            pCopyQuotedGString = &current->program;
            lastStringContext=YY_START;
            BEGIN( CopyGString );
					}
<ReadBody>"/*"{B}*		{ 
					  current->program += yytext ;
					  lastContext = YY_START ;
					  BEGIN( Comment ) ;
					}
<ReadBody>"/*"{BL}		{ 
					  current->program += yytext ;
					  ++yyLineNr ;
					  lastContext = YY_START ;
					  BEGIN( Comment ) ;
					}
<ReadBody>"'"	{
					  current->program += yytext;
  				}
<ReadBody>{CHARLIT} { 
            current->program += yytext; 
          }
<ReadBody>"{"       { 
					  current->program += yytext ;
					  ++curlyCount ;
					}
<ReadBody>"}"{BN}+"typedef"{BN}+	{ //err("ReadBody count=%d\n",curlyCount);
					  lineCount();
  					if ( curlyCount>0 )
					  {
					    current->program += yytext ;
					    --curlyCount ;
					  }
					  else
					  {
					    isTypedef = TRUE;
					    current->endBodyLine = yyLineNr;
					    QCString &cn = current->name;
					    QCString rn = current_root->name.copy();
					    if (!cn.isEmpty() && !rn.isEmpty())
					    {
					      prependScope();
					    }
					    BEGIN( TypedefName );
					  }
					}
<TypedefName>{ID}			{
  					if (current->section == Entry::ENUM_SEC)
					  {
					    current->program+=","; // add field terminator
					  }
  				  current->name=yytext;
					  prependScope();
  					current->args = current->args.simplifyWhiteSpace();
  					current->type = current->type.simplifyWhiteSpace();
					  //printf("Adding compound %s %s %s\n",current->type.data(),current->name.data(),current->args.data());
					  current_root->addSubEntry( current ) ;
					  if (!firstTypedefEntry)
					  {
					    firstTypedefEntry = current;
					  }
					  current = new Entry;
					  initEntry();
					  isTypedef=TRUE; // to undo reset by initEntry()
					  current->systemVerilogType = SystemVerilogTypedef;
  					BEGIN(MemberSpecSkip); 
  				}
<TypedefName>";"			{ /* typedef of anonymous type */
					  current->name.sprintf("@%d",anonCount++);
					  if (current->section == Entry::ENUM_SEC)
					  {
					    current->program+=','; // add field terminator
					  }
					  // add compound definition to the tree
  					current->args = current->args.simplifyWhiteSpace();
  					current->type = current->type.simplifyWhiteSpace();
					  current_root->addSubEntry( current ) ;
					  current = new Entry(*current);
					  unput(';');
					  BEGIN( MemberSpec ) ;
  				}
<MemberSpec>([*&]*{BN}*)*{ID}{BN}*("["[^\]\n]*"]")* { // the [] part could be improved.
  					lineCount();
  					int i=0,l=yyleng,j;
					  while (i<l && (!isId(yytext[i]))) i++;
					  msName = QCString(yytext).right(l-i).stripWhiteSpace();
					  j=msName.find("[");
					  if (j!=-1) 
					  {
					    msArgs=msName.right(msName.length()-j);
					    msName=msName.left(j);
					  }
					  msType=QCString(yytext).left(i);
					  // handle *pName in: typedef { ... } name, *pName;
					  if (firstTypedefEntry) 
					  {
					    if (firstTypedefEntry->spec&Entry::Struct)
					    {
					      msType.prepend("struct "+firstTypedefEntry->name);
					    }
					    else if (firstTypedefEntry->spec&Entry::Union)
					    {
					      msType.prepend("union "+firstTypedefEntry->name);
					    }
					    else if (firstTypedefEntry->section==Entry::ENUM_SEC)
					    {
					      msType.prepend("enum "+firstTypedefEntry->name);
					    }
					    else
					    {
					      msType.prepend(firstTypedefEntry->name);
					    }
					  }
					}
<MemberSpec>[,;]			{
            //printf("current->name=`%s' msName=`%s'\n",current->name.data(),msName.data());
  					if (msName.isEmpty() && !current->name.isEmpty())
					  { 
					    
					    // see if the compound does not have a name or is inside another
					    // annonymous compound. If so we insert a 
					    // special `annonymous' variable.
					    //Entry *p=current_root;
					    Entry *p=current;
					    while (p)
					    {
					      // only look for class scopes, not namespace scopes
					      if ((p->section & Entry::COMPOUND_MASK) && !p->name.isEmpty())
					      {
									//printf("Trying scope `%s'\n",p->name.data());
									int i=p->name.findRev("::");
									int pi = (i==-1) ? 0 : i+2;
									if (p->name.at(pi)=='@')
									{
										// annonymous compound inside -> insert dummy variable name
										//printf("Adding annonymous variable for scope %s\n",p->name.data());
		  							msName.sprintf("@%d",anonCount++); 
										break;
									}
					      }
					      //p=p->parent;
					      if (p==current) p=current_root; else p=p->parent();
					    }
					  }
					  //printf("msName=%s current->name=%s\n",msName.data(),current->name.data());
					  if (!msName.isEmpty() && msName!=current->name) // skip typedef T {} T;
					  {
  					  
  					  Entry *varEntry=new Entry;
					    varEntry->protection = current->protection ;
              varEntry->mtype = current->mtype;
					    varEntry->virt = current->virt;
					    varEntry->stat = current->stat;
					    varEntry->section = Entry::VARIABLE_SEC;
					    varEntry->name = msName.stripWhiteSpace();
					    varEntry->type = current->type.simplifyWhiteSpace()+" ";
					    varEntry->args = msArgs; //current->args.simplifyWhiteSpace();
					    if (isTypedef)
					    {
					      varEntry->type.prepend("typedef ");
					    }
					    varEntry->type+=current->name+msType;
					    varEntry->fileName = yyFileName;
					    varEntry->startLine = yyLineNr;
					    varEntry->doc = current->doc.copy();
					    varEntry->brief = current->brief.copy();
					    varEntry->mGrpId = current->mGrpId;

					    // deep copy group list
					    QListIterator<Grouping> gli(*current->groups);
					    Grouping *g;
					    for (;(g=gli.current());++gli)
					    {
					      varEntry->groups->append(new Grouping(*g));
					    }
					    if (current->sli) // copy special list items
					    {
					      QListIterator<ListItemInfo> li(*current->sli);
					      ListItemInfo *lii;
					      for (li.toFirst();(lii=li.current());++li)
					      {
									varEntry->addSpecialListItem(lii->type,lii->itemId);
					      }
					      //delete current->sli;
					      //current->sli = 0;
					    }
					    current_root->addSubEntry(varEntry);
					  }
					  if (*yytext==';')
					  {
							bool tmpbool = current->systemVerilogType == SystemVerilogTypedef;
					    msType.resize(0);
					    msName.resize(0);
					    msArgs.resize(0);
					    isTypedef=FALSE;
					    firstTypedefEntry=0;
					    current->reset();
					    initEntry();
					    if(tmpbool)
					    {
								//current->systemVerilogType = SystemVerilogTypedef;
					    }
					    BEGIN( FindMembers );
					  }
  				}
<MemberSpec>"="				{ 
  					lastInitializerContext=YY_START;
					  initBracketCount=0;
  					BEGIN(ReadInitializer);
					}
<MemberSpecSkip>","			{ 
						BEGIN(MemberSpec); 
					}
<MemberSpecSkip>";"		        { 
						unput(';'); 
						BEGIN(MemberSpec); 
					}
<ReadBody,ReadPorts>{BN}+	{ 
					  current->program += yytext ;
					  lineCount() ;
					}
<ReadBody>.	{ 
					  current->program += yytext ; 
					}

<FindMembers>")"			{ 
			roundCount--;
			if(roundCount == 0)
			{
				current_root->insidePortList = FALSE;
			}
			}
					
<FindMembers>"("			{ 
            if (!current->name.isEmpty())
					  {
					    current->args = yytext;
					    current->bodyLine = yyLineNr;
					    currentArgumentContext = FuncQual;
					    fullArgString=current->args.copy();
					    copyArgString=&current->args;
					    BEGIN( ReadFuncArgType ) ;
					    //printf(">>> Read function arguments current->argList->count()=%d\n",current->argList->count());
					  }
					  roundCount++;
					}

  /*- Function argument reading rules ---------------------------------------*/

<ReadFuncArgType>[^ \/\r\t\n\)\(\"\'#]+ { 
					  *copyArgString+=yytext;
  					fullArgString+=yytext;
  				}
<CopyArgString>[^\n\\\"\']+		{ 
						*copyArgString+=yytext; 
					  fullArgString+=yytext;
					}
<CopyArgRound>[^\/\n\)\(\"\']+		{ 
  					*copyArgString+=yytext; 
  					fullArgString+=yytext;
  				}
<ReadFuncArgType>{BN}*	{
  					*copyArgString+=" ";
  					fullArgString+=" ";
  					lineCount();
  				}
<ReadFuncArgType,CopyArgRound>\"	{
  					*copyArgString+=*yytext;
  					fullArgString+=*yytext;
					  lastCopyArgStringContext = YY_START;
  					BEGIN( CopyArgString );
  				}
<ReadFuncArgType>"("	{
  					*copyArgString+=*yytext;
  					fullArgString+=*yytext;
  					argRoundCount=0; 
					  lastCopyArgContext = YY_START;
					  BEGIN( CopyArgRound ); 
  				}
<ReadFuncArgType>")"			{ 
  					*copyArgString+=*yytext;
  					fullArgString+=*yytext;
					  stringToArgumentList(fullArgString,current->argList);
            handleParametersCommentBlocks();
					  /* remember the current documentation block, since
					     we could overwrite it with the documentation of
					     a function argument, which we then have to correct later
					     on
					   */
					  BEGIN( currentArgumentContext );
					}
	/* a special comment */
<ReadFuncArgType>("/*"[*!]|"//"[/!])("<"?)	{ 
            if (currentArgumentContext==DefineEnd)
					  {
					    // for defines we interpret a comment
					    // as documentation for the define 
					    int i;for (i=yyleng-1;i>=0;i--)
					    {
					      unput(yytext[i]);
					    }
					    stringToArgumentList(fullArgString,current->argList);
              handleParametersCommentBlocks();
					    BEGIN( currentArgumentContext );
					  }
					  else // not a define
					  {
					    // for functions we interpret a comment
					    // as documentation for the argument
					    fullArgString+=yytext;
					    lastCopyArgChar=0;
					    lastCommentInArgContext=YY_START;
					    if (yytext[1]=='/')
					      BEGIN( CopyArgCommentLine );
					    else
					      BEGIN( CopyArgComment );
					  }
  				}
	/* a non-special comment */
<ReadFuncArgType>"/*"	{
  					lastCContext = YY_START;
					  BEGIN( SkipComment );
  				}
<ReadFuncArgType>"//"	{
  					lastCContext = YY_START;
					  BEGIN( SkipCxxComment );
  				}
	/* `)' followed by a special comment */
<ReadFuncArgType>")"{BN}*("/*"[*!]|"//"[/!])"<"	{
  					lineCount();
            if (currentArgumentContext==DefineEnd)
					  {
					    // for defines we interpret a comment
					    // as documentation for the define 
					    int i;
					    for (i=yyleng-1;i>0;i--)
					    {
					      unput(yytext[i]);
					    }
					    *copyArgString+=*yytext;
					    fullArgString+=*yytext;
					    stringToArgumentList(fullArgString,current->argList);
              handleParametersCommentBlocks();
					    BEGIN( currentArgumentContext );
					  }
					  else
					  {
					    // for functions we interpret a comment
					    // as documentation for the last argument
					    lastCopyArgChar=*yytext;
					    QCString text=&yytext[1];
					    text=text.stripWhiteSpace();
					    lastCommentInArgContext=YY_START;
					    fullArgString+=text;
					    if (text.find("//")!=-1)
					      BEGIN( CopyArgCommentLine );
					    else
					      BEGIN( CopyArgComment );
					  }
  				}
<CopyArgComment>^{B}*"*"+/{BN}+		
<CopyArgComment>[^\n\\\@\*]+		{ 
						fullArgString+=yytext; 
					}
<CopyArgComment>"*/"			{ 
						fullArgString+=yytext; 
  					if (lastCopyArgChar!=0)
					  unput(lastCopyArgChar); 
            BEGIN( lastCommentInArgContext ); 
					}
<CopyArgCommentLine>\n			{ 
						fullArgString+=yytext;
  					yyLineNr++;
  					if (lastCopyArgChar!=0)
					  unput(lastCopyArgChar);
					  BEGIN( lastCommentInArgContext );
  				}
<CopyArgCommentLine>{CMD}("verbatim"|"latexonly"|"htmlonly"|"xmlonly"|"manonly"|"dot"|"code")/[^a-z_A-Z0-9]	{ // verbatim command (which could contain nested comments!)
				    docBlockName=&yytext[1];
  					fullArgString+=yytext;
					  BEGIN(CopyArgVerbatim);
  				}
<CopyArgCommentLine>{CMD}("f$"|"f["|"f{")	        {
				    docBlockName=&yytext[1];
					  if (docBlockName.at(1)=='[')
					  {
							docBlockName.at(1)='}';
					  }
					  if (docBlockName.at(1)=='{')
					  {
							docBlockName.at(1)='}';
					  }
  					fullArgString+=yytext;
  					BEGIN(CopyArgVerbatim);
					}
<CopyArgVerbatim>[\\@]("endverbatim"|"endlatexonly"|"endhtmlonly"|"endxmlonly"|"enddot"|"endcode"|"f$"|"f]"|"f}")/[^a-z_A-Z0-9] { // end of verbatim block
  					fullArgString+=yytext;
				    if (yytext[1]=='f') // end of formula
				    {
							BEGIN(CopyArgCommentLine);
				    }
					  if (&yytext[4]==docBlockName)
					  {
  						BEGIN(CopyArgCommentLine);
					  }
  				}
<CopyArgCommentLine>[^\\\@\n]+		{ 
						fullArgString+=yytext; 
					}
<CopyArgCommentLine>.			{ 
						fullArgString+=*yytext; 
					}
<CopyArgComment,CopyArgVerbatim>\n	{ 
						fullArgString+=*yytext; 
						yyLineNr++; 
					}
<CopyArgComment,CopyArgVerbatim>.	{ 
						fullArgString+=*yytext; 
					}
<CopyArgComment>{CMD}("brief"|"short"){B}+ {
  					warn(yyFileName,yyLineNr,"Warning: Ignoring %cbrief command inside argument documentation",*yytext);
            fullArgString+=' ';
					}
<CopyArgRound>"("			{
  					argRoundCount++;
					  *copyArgString+=*yytext;
					  fullArgString+=*yytext;
  				}
<CopyArgRound>")"			{
					  *copyArgString+=*yytext;
					  fullArgString+=*yytext;
					  if (argRoundCount>0) 
					    argRoundCount--;
					  else 
					    BEGIN( lastCopyArgContext );
  				}
<CopyArgString>\\.	{
  					*copyArgString+=yytext;
  					fullArgString+=yytext;
  				}
<CopyArgString>\"			{
  					*copyArgString+=*yytext;
  					fullArgString+=*yytext;
					  BEGIN( lastCopyArgStringContext );
  				}
<ReadFuncArgType,CopyArgRound>{CHARLIT}     { 
  					*copyArgString+=yytext; 
  					fullArgString+=yytext; 
					}
<ReadFuncArgType,CopyArgRound>\'     { 
  					*copyArgString+=yytext; 
  					fullArgString+=yytext; 
  				}
<ReadFuncArgType,CopyArgString,CopyArgRound>\n  { 
  					yyLineNr++; 
					  *copyArgString+=*yytext; 
					  fullArgString+=*yytext; 
					}
<ReadFuncArgType,CopyArgString,CopyArgRound>.	  { 
  					*copyArgString+=*yytext; 
  					fullArgString+=*yytext; 
					}

  /*------------------------------------------------------------------------*/


<FuncRound>"("				{ current->args += *yytext ;
					  ++roundCount ;
					}
<FuncRound>")"                          { 
						current->args += *yytext ; 
					  if ( roundCount )
					    --roundCount ;
				    else
					    BEGIN( FuncQual ) ;
					}
<FuncQual>[{:;,]                        {
            unput(*yytext); 
            BEGIN( Function );
					}
<FuncQual>{BN}*"const"{BN}*       	{ // const member function
  					lineCount() ; 
  					current->args += " const "; 
					  current->argList->constSpecifier=TRUE;
					}
<FuncQual>{BN}*"="{BN}*"0"{BN}*  	{ // pure virtual member function
  					lineCount() ; 
					  current->args += " = 0"; 
					  current->virt = Pure; 
					  current->argList->pureSpecifier=TRUE;
					}
<FuncRound>{BN}*","{BN}*	{ 
  					lineCount() ; 
					  current->args += ", " ; 
					}
<FuncQual,FuncRound>{BN}+   	{ 
  					lineCount() ; 
					  current->args += ' ' ; 
					}
<Function,FuncQual,FuncRound>"#" { 
  					lastCPPContext = YY_START;
  					BEGIN(SkipCPP);
					}	
<FuncQual>"="				{ 
            // typically an initialized function pointer
  					lastInitializerContext=YY_START;
					  initBracketCount=0;
  					BEGIN(ReadInitializer);
  				}
<FuncQual>{ID}				{ // typically a K&R style C function
						current->args += yytext;
  				}
<FuncQual,FuncRound>.		{ 
					  current->args += *yytext; 
					}
<Function>"("				{ 
					  current->type += current->name ;
					  current->name  = current->args ;
					  current->args  = yytext ;
					  roundCount=0;
					  BEGIN( FuncRound ) ;
					}
<Function>[;{,]				{ 
					  current->name=current->name.simplifyWhiteSpace();
  					current->type=current->type.simplifyWhiteSpace();
					  current->args=removeRedundantWhiteSpace(current->args);
					  // was: current->args.simplifyWhiteSpace();
					  current->fileName = yyFileName;
					  current->startLine = yyLineNr;
					  if (*yytext!=';' || (current_root->section&Entry::COMPOUND_MASK) )
					  {
					    int tempArg=current->name.find('<');
					    QCString tempName;
					    static QRegExp re("operator[^a-z_A-Z0-9]");
					    if (tempArg==-1) tempName=current->name; else tempName=current->name.left(tempArg);
					    if (current->type.left(8)=="typedef ")
					    {
					      //printf("Scanner.l: found in class variable: `%s' `%s' `%s'\n",
					      //   current->type.data(),current->name.data(),current->args.data());
					      current->section = Entry::VARIABLE_SEC ;
					    }
					    else	      
					    {
					      //printf("Scanner.l: found in class function: `%s' `%s' `%s'\n",
					      //   current->type.data(),current->name.data(),current->args.data());
					      current->section = Entry::FUNCTION_SEC ;
			          current->proto = *yytext==';';
					    }
					  }
					  else // a global function prototype or function variable
					  {
					    static QRegExp re("([^)]*\\*[^)]*)"); // (...*...)
					    //printf("Scanner.l: prototype? type=`%s' name=`%s' args=`%s'\n",current->type.data(),current->name.data(),current->args.data());
					    if (!current->type.isEmpty() && (current->type.find(re,0)!=-1 || current->type.left(8)=="typedef "))
					    {
					      //printf("Scanner.l: found function variable!\n");
					      current->section = Entry::VARIABLE_SEC;
					    }
					    else
					    {
					      //printf("Scanner.l: found prototype\n");
					      current->section = Entry::FUNCTION_SEC;
					      current->proto = TRUE;
					    }
					  }
					  //printf("Adding entry `%s'\n",current->name.data());
					  previous = current;
					  current_root->addSubEntry(current);
					  current = new Entry ;
					  initEntry();
					  lastCurlyContext = FindMembers;
					  if ( *yytext == ',' )
					  {
					    current->type = previous->type.data();
					  }
					  if ( *yytext == '{' )
					  {
					    if (current_root->section & Entry::COMPOUND_MASK)
					    {
						  previous->spec |= Entry::Inline;
					    }
					    //addToBody(yytext);
					    curlyCount=0;
					    BEGIN( SkipCurly ) ;
					  }
					  else
					  {
					    if (previous->section!=Entry::VARIABLE_SEC)
								//previous->bodyLine=-1; // a function/member declaration
								BEGIN( FindMembers ) ;
					  }
					}
<SkipCurly>"begin"({B}*":")?				{ 	
  					++beginCount; 
  					//printf("SkipCurly 1: %s\n", yytext);
					}
<SkipCurly>"case"	{
					  ++beginCount;
					  //printf("SkipCurly 2: %s\n", yytext);
					}
<SkipCurly>"endcase"	{
						--beginCount;
						//printf("SkipCurly 3: %s\n", yytext);
					}
<SkipCurly>"end"			{ 
					//printf("SkipCurly 4: %s\n", yytext);
  					if( beginCount > 1)
					  {
					    --beginCount ;
					  }
					  else
					  {
							--beginCount;
					    if (!Config_getBool("HIDE_IN_BODY_DOCS") && !current->doc.isEmpty())
					    {
					      // copy documentation found inside the body
					      // to the previous item
					      if (previous->inbodyLine==-1)
					      {
					        previous->inbodyLine = current->docLine;
					        previous->inbodyFile = current->docFile;
					      }
					      //printf("*** inbodyDocs+=%s\n",current->doc.data());
					      previous->inbodyDocs += current->doc;
					      current->doc.resize(0);
					    }
					    if (current->sli) // copy special list items
					    {
					      QListIterator<ListItemInfo> li(*current->sli);
					      ListItemInfo *lii;
					      for (li.toFirst();(lii=li.current());++li)
					      {
									previous->addSpecialListItem(lii->type,lii->itemId);
					      }
					      delete current->sli;
					      current->sli = 0;
					    }
					    //previous->endBodyLine=yyLineNr;
					    int i;
					    for (i=yyleng-1;i>=0;i--)
					    {
					      unput(yytext[i]);
					    }
					    BEGIN( lastCurlyContext ) ;
					  }
					}
<SkipCurly>"}"{BN}*("/*!"|"/**"|"//!"|"///")"<" { 
					  lineCount();
  					if ( curlyCount )
					  {
					    //addToBody(yytext);
					    --curlyCount ;
					  }
					  else
					  {
					    current->endBodyLine=yyLineNr;

					    tempEntry = current; // temporarily switch to the previous entry
					    current = previous;

					    docBlockContext   = SkipCurlyEndDoc;
					    docBlockInBody    = FALSE;
					    docBlockAutoBrief = ( yytext[yyleng-2]=='*' && Config_getBool("JAVADOC_AUTOBRIEF") ) || ( yytext[yyleng-2]=='!' && Config_getBool("QT_AUTOBRIEF") );
					    docBlock.resize(0);
					    docBlockTerm = '}';
					    if (yytext[yyleng-3]=='/')
					    {
					      startCommentBlock(TRUE);
					      BEGIN( DocLine );
					    }
					    else
					    {
					      startCommentBlock(FALSE);
					      BEGIN( DocBlock );
					    }
					  }
					}
<SkipCurlyEndDoc>"}"{BN}*("/*!"|"/**"|"//!"|"///")"<" { // desc is followed by another one
					  docBlockContext   = SkipCurlyEndDoc;
					  docBlockInBody    = FALSE;
					  docBlockAutoBrief = ( yytext[yyleng-2]=='*' && Config_getBool("JAVADOC_AUTOBRIEF") ) || ( yytext[yyleng-2]=='!' && Config_getBool("QT_AUTOBRIEF") );
					  docBlock.resize(0);
					  docBlockTerm = '}';
					  if (yytext[yyleng-3]=='/')
					  {
					    startCommentBlock(TRUE);
					    BEGIN( DocLine );
					  }
					  else
					  {
					    startCommentBlock(FALSE);
					    BEGIN( DocBlock );
					  }
  				}
<SkipCurlyEndDoc>"}"			{
					  current = tempEntry;
  					BEGIN( lastCurlyContext );
  				}
<SkipCurly>\"			        { 
  					lastStringContext=SkipCurly;
  				  BEGIN( SkipString ); 
					}
<SkipCurly>\n			{
  					yyLineNr++;
  				  //addToBody(yytext);
  				}
  				
<SkipCurly>[^\n#"'@\\/{}() \t]+	{
  				}
  
<SkipCurly>"/*"	{
  					lastCContext = YY_START;
					  BEGIN(SkipComment);
  				}
<SkipCurly>"//"  {
  					lastCContext = YY_START;
					  BEGIN(SkipCxxComment);
  				}
<SkipCurly>{CHARLIT}	{
					}
<SkipCurly>\'	{
					}
<SkipCurly>.	{ 
					}
<SkipString>\\.		{ }
<SkipString>\"				{ 
  					BEGIN( lastStringContext ); 
					}
<SkipString>"/*"|"*/"|"//" { }
<SkipString>\n		{
  					yyLineNr++;
  				}
<SkipString>.		{ }
<CompoundName>";"			{ 
					  current->section = Entry::EMPTY_SEC ;
					  current->type.resize(0) ;
					  current->name.resize(0) ;
					  current->args.resize(0) ;
					  current->argList->clear();
					  BEGIN( FindMembers ) ;
					}
<CompoundName>"packed"|"tagged"		{
						current_root->insidePortList = FALSE;
						lineCount();
					  BEGIN( ClassVar );
					}
<CompoundName>{ID}		{ 
					  current->name = yytext ;
					  if(isTypedef)
						{	
							current_root->insidePortList = FALSE;
						}
					  lineCount();
					  if (current->spec & Entry::Protocol)
					  {
					    current->name += "-p";
					  }
					  if ((current->section & Entry::Protocol) || current->section == Entry::OBJCIMPL_SEC)
					  {
					    unput('{'); // fake start of body
					  }
					  BEGIN( ClassVar );
					}
<ClassVar>{ID}{BN}*/"("		{
  					addType(current);
					  current->name = yytext;
					  current->name = current->name.stripWhiteSpace();
					  lineCount();
  					BEGIN( FindMembers );
  				}
<ClassVar>","				{
 					  if (isTypedef)
					  {
					    // multiple types in one typedef
					    unput(',');
					    current->type.prepend("typedef ");
					    BEGIN(FindMembers);
					  }
					  else
					  { 
  						// Multiple class forward declaration
					  }
  				}
<ClassVar>{ID}				{
					  if (current->section == Entry::ENUM_SEC)
					  { // found "enum a b" -> variable
					    current->section = Entry::VARIABLE_SEC ;
					  }
					  current->type += ' ' ;
					  current->type += current->name ;
					  current->name = yytext ;
  				}
<ClassVar>[;=*&]			{
    				current->fileName = yyFileName ;
					  current->startLine = yyLineNr ;
					  current->name = removeRedundantWhiteSpace(current->name);
					  current->insidePortList = FALSE;
					  BEGIN( ReadBody );
    			}
<ClassVar>{B}*"#("{B}* {
					  //printf("classvar: %s\n", yytext);
            current->fileName = yyFileName ;
					  current->startLine = yyLineNr ;
					  current->name = removeRedundantWhiteSpace(current->name);
					  curlyCount=0;				
						int i;
						for (i=yyleng-1;i>=0;i--)
					  {
					    unput(yytext[i]);
					  }
					  BEGIN( ReadBody ) ;
					}
<CompoundName,ClassVar>{B}*"("{B}*	{ 
            current->fileName = yyFileName ;
					  current->startLine = yyLineNr ;
					  current->name = removeRedundantWhiteSpace(current->name);
					  if (current->name.isEmpty() && !isTypedef) // anonymous compound
					  {
							current->name.sprintf("@%d",anonCount++);
					  }
					  curlyCount=0;
					  BEGIN( ReadBody ) ;
					}
<Comment>{BN}+				{ 
						current->program += yytext ;
					  lineCount() ;
					}
<Comment>"/*"				{ 
						current->program += yytext ; 
					} 
<Comment>"//"				{ 
						current->program += yytext ; 
					}
<Comment>{CMD}("code"|"verbatim")	{
            insideCode=TRUE;
  					current->program += yytext ;
  				}
<Comment>{CMD}("endcode"|"endverbatim")	{
            insideCode=FALSE;
  					current->program += yytext ;
  				}
<Comment>[^ \.\t\r\n\/\*]+		{ 
						current->program += yytext ; 
					}
<Comment>"*/"				{ 
						current->program += yytext ;
					  if (!insideCode) 
							BEGIN( lastContext ) ;
					}
<Comment>.				{ 
						current->program += *yytext ; 
					}

<FindMembers,FindFields,MemberSpec,FuncQual,SkipCurly,ClassVar,GenerateDoFor,GenerateDoIf,GenerateDoElse,EnumName>("//"{B}*)?"/*!" { 
  					//printf("Start doc block at %d\n",yyLineNr);
  					if (YY_START!=SkipCurly)
					  {
					    current->doc.resize(0);
					    current->docLine = yyLineNr;
					    current->docFile = yyFileName;
					  }
					  if (current_root->section & Entry::SCOPE_MASK)
            {
					    current->inside = current_root->name+"::";
            }
					  docBlockContext   = YY_START;
					  docBlockInBody    = YY_START==SkipCurly;
					  docBlockAutoBrief = Config_getBool("QT_AUTOBRIEF");
					  docBlock.resize(0);
					  if (docBlockAutoBrief)
					  {
					    current->briefLine = yyLineNr;
					    current->briefFile = yyFileName;
					  }
					  if (!docBlockInBody)
					  {
					    current->doc.resize(0);
					    if (docBlockAutoBrief)
					    {
					      current->brief.resize(0);
					    }
					  }
					  startCommentBlock(FALSE);
					  BEGIN( DocBlock );
					}
<FindMembers,FindFields,MemberSpec,FuncQual,SkipCurly,ClassVar,GenerateDoFor,GenerateDoIf,GenerateDoElse,EnumName,GenerateFor,GenerateIf,GenerateElse>("//"{B}*)?"/**"/[^/*] {
					  //printf("Found comment block at %s:%d\n",yyFileName,yyLineNr);
					  if (current_root->section & Entry::SCOPE_MASK)
            {
					    current->inside = current_root->name+"::";
						}
					  current->docLine = yyLineNr;
					  current->docFile = yyFileName;
					  docBlockContext = YY_START;
					  docBlockInBody  = YY_START==SkipCurly;
					  docBlockAutoBrief = Config_getBool("JAVADOC_AUTOBRIEF");
					  docBlock.resize(0);
					  if (docBlockAutoBrief)
					  {
					    current->briefLine = yyLineNr;
					    current->briefFile = yyFileName;
					  }
					  if (!docBlockInBody)
					  {
					    current->doc.resize(0);
					    if (docBlockAutoBrief)
					    {
					      current->brief.resize(0);
					    }
					  }
					  startCommentBlock(FALSE);
					  BEGIN( DocBlock );
  				}
<FindMembers,FindFields,MemberSpec,SkipCurly,FuncQual,ClassVar,GenerateDoFor,GenerateDoIf,GenerateDoElse,EnumName>"//!" { 
            if (YY_START!=SkipCurly)
					  {
  					  current->brief.resize(0);
					    current->briefFile=yyFileName;
					    current->briefLine=yyLineNr;
					  }
					  if (current_root->section & Entry::SCOPE_MASK)
            {
					    current->inside = current_root->name+"::";
              //if (current->mGrpId!=DOX_NOGROUP)
              //{
              //  memberGroupInside = current->inside.copy();
              //}
            }
					  docBlockContext   = YY_START;
					  docBlockInBody    = YY_START==SkipCurly;
					  docBlockAutoBrief = FALSE;
					  docBlock.resize(0);
					  startCommentBlock(TRUE);
					  BEGIN( DocLine );
					}
<FindMembers,FindFields,MemberSpec,SkipCurly,FuncQual,ClassVar,GenerateDoFor,GenerateDoIf,GenerateDoElse,EnumName>"///"/[^/] { 
			printf("SkipCurly: %s\n", yytext);
            if (YY_START!=SkipCurly)
			{
  			  current->brief.resize(0);
			  current->briefFile=yyFileName;
			  current->briefLine=yyLineNr;
			}
			if (current_root->section & Entry::SCOPE_MASK)
			{
			  current->inside = current_root->name+"::";
              //if (current->mGrpId!=DOX_NOGROUP)
              //{
              //  memberGroupInside = current->inside.copy();
              //}
            }
			docBlockContext   = YY_START;
			docBlockInBody    = YY_START==SkipCurly;
			docBlockAutoBrief = FALSE;
			docBlock.resize(0);
			startCommentBlock(TRUE);
			BEGIN( DocLine );
		}

 /**********************************************************************************/
 /******************** Documentation block related rules ***************************/
 /**********************************************************************************/

 /* ---- Single line comments ------ */
<DocLine>[^\n]*"\n"[ \t]*"//"[/!]       { // continuation of multiline C++-style comment
			//printf("DocLine 1: %s\n", yytext);
						docBlock+=yytext;
          	docBlock.resize(docBlock.size() - 3);
						yyLineNr++;
          }
<DocLine>[^\n]*/"\n"			{ // whole line
			//printf("DocLine 2: %s\n", yytext);
  					docBlock+=yytext;
					  handleCommentBlock(docBlock,TRUE);
					  BEGIN( docBlockContext );
  				}

 /* ---- Comments blocks ------ */

<DocBlock>"*"*"*/"			{ // end of comment block
  				  handleCommentBlock(docBlock,FALSE);
					  BEGIN(docBlockContext);
  				}
<DocBlock>^{B}*("//")?{B}*"*"+/[^//]	{ // start of a comment line
  				}
<DocBlock>^{B}*("//"){B}*		{ // strip embedded C++ comments if at the start of a line
  					}
<DocBlock>"//"				{ // slashes in the middle of a comment block
  					docBlock+=yytext;
  				}
<DocBlock>"/*"				{ // start of a new comment in the 
            // middle of a comment block
  					docBlock+=yytext;
  				}
<DocBlock>("@@"|"\\\\"){ID}/[^a-z_A-Z0-9] { // escaped command
  					docBlock+=yytext;
  				}
<DocBlock>{CMD}("f$"|"f["|"f{")	        {
            docBlock+=yytext;
				    docBlockName=&yytext[1];
					  if (docBlockName.at(1)=='{')
					  {
							docBlockName.at(1)='}';
					  }
  					BEGIN(DocCopyBlock);
          }
<DocBlock>"<"{PRE}">"			{
            docBlock+=yytext;
				    docBlockName="<pre>";
  					BEGIN(DocCopyBlock);
  				}
<DocBlock>{CMD}("verbatim"|"latexonly"|"htmlonly"|"xmlonly"|"manonly"|"dot"|"code")/[^a-z_A-Z0-9]	{ // verbatim command (which could contain nested comments!)
            docBlock+=yytext;
				    docBlockName=&yytext[1];
  					BEGIN(DocCopyBlock);
  				}
<DocBlock>[^@*\/\\\n]+			{ // any character that isn't special
  					docBlock+=yytext;
  				}
<DocBlock>\n				{ // newline
  					yyLineNr++;
					  docBlock+=*yytext;
  				}
<DocBlock>.				{ // command block
					  docBlock+=*yytext;
  				}

 /* ---- Copy verbatim sections ------ */

<DocCopyBlock>"</"{PRE}">"		{ // end of a <pre> block
  					docBlock+=yytext;
					  if (docBlockName=="<pre>")
					  {
  						BEGIN(DocBlock);
					  }
  				}
<DocCopyBlock>"</"{CODE}">"		{ // end of a <code> block
  					docBlock+=yytext;
					  if (docBlockName=="<code>")
					  {
  						BEGIN(DocBlock);
					  }
  				}
<DocCopyBlock>[\\@]("f$"|"f]"|"f}")     {
  					docBlock+=yytext;
					  BEGIN(DocBlock);
  				}
<DocCopyBlock>[\\@]("endverbatim"|"endlatexonly"|"endhtmlonly"|"endxmlonly"|"enddot"|"endcode")/[^a-z_A-Z0-9] { // end of verbatim block
  					docBlock+=yytext;
					  if (&yytext[4]==docBlockName)
					  {
  						BEGIN(DocBlock);
					  }
  				}
<DocCopyBlock>^{B}*"*"+/{BN}+		{ // start of a comment line
  					if (docBlockName=="verbatim")
					  {
					    REJECT;
					  }
  				}
<DocCopyBlock>[^\<@/*\]\$\\\n]+		{ // any character that is not special
  					docBlock+=yytext;
  				}
<DocCopyBlock>"/*"|"*/"|"//"		{
  					docBlock+=yytext;
					}
<DocCopyBlock>\n			{ // newline
  					docBlock+=*yytext;
					  yyLineNr++;
  				}
<DocCopyBlock>.				{ // any other character
  					docBlock+=*yytext;
  				}
<DocCopyBlock><<EOF>>			{
  					warn(yyFileName,yyLineNr,
					    "Warning: reached end of file while inside a %s block!\n"
					    "The command that should end the block seems to be missing!\n",
					    docBlockName.data());
  					yyterminate();
  				}


    /* ------------- Prototype parser -------------- */

<Prototype>"("			{
  					current->args+=*yytext;
					  currentArgumentContext = PrototypeQual;
					  fullArgString = current->args.copy();
					  copyArgString = &current->args;
					  BEGIN( ReadFuncArgType ) ;
  				}
<Prototype>"("({ID}"::")*({B}*"*")+	{
  					current->type+=current->name+yytext;
					  current->name.resize(0);
  					BEGIN( PrototypePtr );
  				}
<PrototypePtr>{SCOPENAME}		{
  					current->name+=yytext;
  				}
<PrototypePtr>")"			{
  					current->type+=')';
  					BEGIN( Prototype );
  				}
<PrototypeQual>"{"			{
  					BEGIN( PrototypeSkipLine);
  				}
<PrototypeQual>{B}*"const"{B}*    	{ 
  					current->args += " const "; 
					  current->argList->constSpecifier=TRUE;
					}
<PrototypeQual>{B}*"="{B}*"0"{B}*	{ 
					  current->args += " = 0"; 
					  current->virt = Pure; 
					  current->argList->pureSpecifier=TRUE;
					}
<Prototype,PrototypeQual>.	{
  					current->name += *yytext;
  				}
<PrototypeSkipLine>.			{
  				}


  /* ------------ Generic rules -------------- */


<SkipCxxComment>.*/\n			{ 
					  BEGIN( lastCContext ) ;
					}
<SkipComment>[^\*\n]+
<*>\n					{ 
						//printf("NEW LINE FOR DEBUG!\n");
						yyLineNr++ ; }
<*>\"					{
					}

<*>\'					{
  					}
<*>\"					{
  					}
<*>.
<SkipComment>"//"|"/*"
<*>"/*"					{ 
						lastCContext = YY_START ;
					  BEGIN( SkipComment ) ;
					}
<SkipComment>{B}*"*/"			{
						BEGIN( lastCContext ) ; }
<*>"//"				        {	
  					lastCContext = YY_START ;
					  BEGIN( SkipCxxComment ) ;
					}
%%

//----------------------------------------------------------------------------

static void startCommentBlock(bool brief)
{
  if (brief)
  {
    current->briefFile = yyFileName;
    current->briefLine = yyLineNr;
  }
  else
  {
    current->docFile = yyFileName;
    current->docLine = yyLineNr;
  }
}
  
//----------------------------------------------------------------------------

static void newEntry()
{
  current_root->addSubEntry(current);
  previous = current;
  current = new Entry ;
  initEntry();
}

static void handleCommentBlock(const QCString &doc,bool brief)
{
  static bool hideInBodyDocs = Config_getBool("HIDE_IN_BODY_DOCS");
  int position=0;
  bool needsEntry=FALSE;
  if (docBlockInBody && hideInBodyDocs) return;
  //printf("parseCommentBlock [%s]\n",doc.data());
  while (parseCommentBlock(
	g_thisParser,
	docBlockInBody ? previous : current,
	doc,        // text
	yyFileName, // file
	brief ? current->briefLine : current->docLine,   // line of block start
	docBlockInBody ? FALSE : brief, 
	docBlockInBody ? FALSE : docBlockAutoBrief,
	docBlockInBody,
	protection,
        position,
        needsEntry
        )
     ) 
  {
    //printf("parseCommentBlock position=%d [%s]\n",position,doc.data()+position);
    if (needsEntry) newEntry();
  }
  if (needsEntry)
  {
    newEntry();
  }

  if (docBlockTerm)
  {
    unput(docBlockTerm);
    docBlockTerm=0;
  }
}

static void handleParametersCommentBlocks()
{
  //printf(">>>>>>> handleParametersCommentBlocks()\n");
  ArgumentListIterator ali(*current->argList);
  Argument *a;
  for (ali.toFirst();(a=ali.current());++ali)
  {
    //printf("    Param %s docs=%s\n",a->name.data(),a->docs.data());
    if (!a->docs.isEmpty())
    {
      int position=0;
      bool needsEntry;

      // save context
      QCString orgDoc   = current->doc;
      QCString orgBrief = current->brief;
      int orgDocLine    = current->docLine;
      int orgBriefLine  = current->briefLine;

      current->doc.resize(0);
      current->brief.resize(0);

      //printf("handleParametersCommentBlock [%s]\n",doc.data());
      while (parseCommentBlock(
	     g_thisParser,
	     current,
	     a->docs,            // text
	     yyFileName,         // file
	     current->docLine,   // line of block start
	     FALSE, 
	     FALSE,
	     FALSE,
	     protection,
	     position,
	     needsEntry
	    )
	  ) 
      {
	//printf("handleParametersCommentBlock position=%d [%s]\n",position,doc.data()+position);
	if (needsEntry) newEntry();
      }
      if (needsEntry)
      {
	newEntry();
      }
      a->docs = current->doc;

      // restore context
      current->doc       = orgDoc;
      current->brief     = orgBrief;
      current->docLine   = orgDocLine;
      current->briefLine = orgBriefLine;
    }
  }
}


//----------------------------------------------------------------------------

static void parseCompounds(Entry *rt)
{
  //printf("parseCompounds(%s)\n",rt->name.data());
  g_inputFromFile = FALSE;
  EntryListIterator eli(*rt->children());
  Entry *ce;
  for (;(ce=eli.current());++eli)
  {
    if (!ce->program.isEmpty())
    {
      //printf("-- %s ---------\n%s\n---------------\n",
      //  ce->name.data(),ce->program.data());
      // init scanner state
      inputString = ce->program;
      inputPosition = 0;
      svscanYYrestart( svscanYYin ) ;
      if (ce->section==Entry::ENUM_SEC)
				BEGIN( FindFields ) ;
      else
				BEGIN( FindMembers ) ;
      current_root = ce ;
      yyFileName = ce->fileName;
      yyLineNr = ce->startLine ;
      //printf("---> Inner block starts at line %d objC=%d\n",yyLineNr,insideObjC);
      //current->reset();
      if (current) delete current;
      current = new Entry;
      gstat = FALSE;
      int ni=ce->name.findRev("::"); if (ni==-1) ni=0; else ni+=2;
      // set default protection based on the compound type
      if( ce->section==Entry::CLASS_SEC ) // class
      {
				if (ce->spec&(Entry::Interface | Entry::Ref | Entry::Value | Entry::Struct | Entry::Union))
				{
					if (ce->objc)
					{
						current->protection = protection = Protected ;
					}
					else
					{
						current->protection = protection = Public ;
					}
				}
				else 
				{
						current->protection = protection = Private ;
				}
      }
      else if (ce->section == Entry::ENUM_SEC ) // enum
      {
				current->protection = protection = ce->protection;
      }
      else if (!ce->name.isEmpty() && ce->name.at(ni)=='@') // unnamed union or namespace
      {
				current->protection = protection = ce->protection;
      }
      else // named struct, union, protocol, category
      {
				current->protection = protection = Public ;
      }
      mtype = Method;
      virt = Normal;
      //printf("name=%s current->stat=%d gstat=%d\n",ce->name.data(),current->stat,gstat);

      //memberGroupId = DOX_NOGROUP;
      groupEnterCompound(yyFileName,yyLineNr,ce->name);
      
      svscanYYlex() ;
      g_lexInit=TRUE;
      //forceEndGroup();

      groupLeaveCompound(yyFileName,yyLineNr,ce->name);
      
      delete current; current=0;
      ce->program.resize(0);
    }
    parseCompounds(ce);
  }
}

//----------------------------------------------------------------------------

static void parseMain(const char *fileName,const char *fileBuf,Entry *rt)
{
  initParser();
  //g_inputFromFile = TRUE;

  inputString = fileBuf;
  inputPosition = 0;
  g_inputFromFile = FALSE;

  //anonCount     = 0;  // don't reset per file
  protection    = Public;
  mtype         = Method;
  gstat         = FALSE;
  virt          = Normal;
  current_root  = rt;
  inputFile.setName(fileName);
  if (inputFile.open(IO_ReadOnly))
  {
    yyLineNr= 1 ; 
    yyFileName = fileName;
    msg("Parsing file %s...\n",yyFileName.data());
    current_root  = rt ;
    initParser();
    groupEnterFile(yyFileName,yyLineNr);
    current       = new Entry;
    //printf("current=%p current_root=%p\n",current,current_root);
    int sec=guessSection(yyFileName);
    if (sec)
    {
      current->name    = yyFileName;
      current->section = sec;
      current_root->addSubEntry(current);
      current          = new Entry;
    }
    current->reset();
    svscanYYrestart( svscanYYin );
    BEGIN( FindMembers );

    svscanYYlex();
    g_lexInit=TRUE;

    if (YY_START==Comment)
    {
      warn(yyFileName,yyLineNr,"File ended in the middle of a comment block! Perhaps a missing \\endcode?");
    }

    //forceEndGroup();
    groupLeaveFile(yyFileName,yyLineNr);

    rt->program.resize(0);
    delete current; current=0;

    parseCompounds(rt);

    inputFile.close();

    anonNSCount++;
  }
}

//----------------------------------------------------------------------------

static void parsePrototype(const QCString &text)
{
  //printf("**** parsePrototype(%s) begin\n",text.data());
  if (text.isEmpty()) 
  {
    warn(yyFileName,yyLineNr,"Empty prototype found!");
    return;
  }

  const char *orgInputString;
  int orgInputPosition;
  YY_BUFFER_STATE orgState;
  bool orgInputFromFile;
  
  // save scanner state
  orgState = YY_CURRENT_BUFFER;
  yy_switch_to_buffer(yy_create_buffer(svscanYYin, YY_BUF_SIZE));
  orgInputString = inputString; 
  orgInputPosition = inputPosition;
  orgInputFromFile = g_inputFromFile;

  // set new string
  inputString = text;
  inputPosition = 0;
  g_inputFromFile = FALSE;
  svscanYYrestart( svscanYYin );
  BEGIN(Prototype);
  svscanYYlex();
  g_lexInit=TRUE;

  current->name = current->name.stripWhiteSpace();
  if (current->section == Entry::MEMBERDOC_SEC && current->args.isEmpty())
    current->section = Entry::VARIABLEDOC_SEC;

  // restore original scanner state
  YY_BUFFER_STATE tmpState = YY_CURRENT_BUFFER;
  yy_switch_to_buffer(orgState);
  yy_delete_buffer(tmpState);
  inputString = orgInputString; 
  inputPosition = orgInputPosition;
  g_inputFromFile = orgInputFromFile;

  //printf("**** parsePrototype end\n");
}

void svscanFreeScanner()
{
#if defined(YY_FLEX_SUBMINOR_VERSION)
  if (g_lexInit)
  {
    svscanYYlex_destroy();
  }
#endif
}

//----------------------------------------------------------------------------

void SystemVerilogLanguageScanner::parseInput(const char *fileName,const char *fileBuf,Entry *root)
{
  g_thisParser = this;
  ::parseMain(fileName,fileBuf,root);
}

void SystemVerilogLanguageScanner::parseCode(CodeOutputInterface & codeOutIntf,
                   const char * scopeName,
                   const QCString & input,
                   bool isExampleBlock,
                   const char * exampleName,
                   FileDef * fileDef,
                   int startLine,
                   int endLine,
                   bool inlineFragment,
		   MemberDef *memberDef
                  )
{
  ::parseSystemVerilogCode(codeOutIntf,scopeName,input,isExampleBlock,exampleName,
               fileDef,startLine,endLine,inlineFragment,memberDef);
}

bool SystemVerilogLanguageScanner::needsPreprocessing(const QCString &extension)
{
  return TRUE;
}

void SystemVerilogLanguageScanner::resetCodeParserState()
{
  ::resetSystemVerilogCodeParserState();
}

void SystemVerilogLanguageScanner::parsePrototype(const char *text)
{
  ::parsePrototype(text);
}

//----------------------------------------------------------------------------

#if !defined(YY_FLEX_SUBMINOR_VERSION) 
//----------------------------------------------------------------------------
extern "C" { // some bogus code to keep the compiler happy
  void svscannerYYdummy() { yy_flex_realloc(0,0); } 
}
#endif

