/******************************************************************************
 *
 * 
 *
 *
 * Copyright (C) 1997-2007 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */

#include <stdlib.h>

#include "svvalue.h"
#include "genexp.h"

SVValue parseSVOctal()
{
  long val = 0;
  for (const char *p = g_genStrToken.data(); *p != 0; p++)
  {
    if (*p >= '0' && *p <= '7') val = val * 8 + *p - '0';
  }
  return SVValue(val);
}

SVValue parseSVBinary()
{
  long val = 0;
  for (const char *p = g_genStrToken.data(); *p != 0; p++)
  {
    if (*p >= '0' && *p <= '1') val = val * 2 + *p - '0';
  }
  return SVValue(val);	
}

SVValue parseSVDecimal()
{
  long val = 0;
	//printf("parseDecimal: %s\n", g_genStrToken.data());
  for (const char *p = g_genStrToken.data(); *p != 0; p++)
  {
    if (*p >= '0' && *p <= '9') val = val * 10 + *p - '0';
  }
  return SVValue(val);
}

SVValue parseSVHexadecimal()
{
  long val = 0;
  for (const char *p = g_genStrToken.data(); *p != 0; p++)
  {
    if      (*p >= '0' && *p <= '9') val = val * 16 + *p - '0';
    else if (*p >= 'a' && *p <= 'f') val = val * 16 + *p - 'a' + 10;
    else if (*p >= 'A' && *p <= 'F') val = val * 16 + *p - 'A' + 10;
  }
  //printf("parseHexadecimal %s->%x\n",g_strToken.data(),val);
  return SVValue(val);
}

SVValue parseSVCharacter() // does not work for '\n' and the alike 
{
  if (g_genStrToken[1]=='\\')
  {
    switch(g_genStrToken[2])
    {
      case 'n':  return SVValue((long)'\n');
      case 't':  return SVValue((long)'\t');
      case 'v':  return SVValue((long)'\v');
      case 'b':  return SVValue((long)'\b');
      case 'r':  return SVValue((long)'\r');
      case 'f':  return SVValue((long)'\f');
      case 'a':  return SVValue((long)'\a');
      case '\\': return SVValue((long)'\\');
      case '?':  return SVValue((long)'\?');
      case '\'': return SVValue((long)'\'');
      case '"':  return SVValue((long)'"');
      case '0':  return parseSVOctal();
      case 'x': 
      case 'X':  return parseSVHexadecimal();
      default:   printf("Invalid escape sequence %s found!\n",g_genStrToken.data()); 
                 return SVValue(0L); 
    }
  }
  return SVValue((long)g_genStrToken[1]);
}

SVValue parseSVFloat()
{
	QCString tmpString;
  for (const char *p = g_genStrToken.data(); *p != 0; p++)
  {
		if(*p != '_') tmpString += *p;
  }
  return SVValue(atof(tmpString));
}
